""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: switch_controller/security_policy/x802_1x
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class X8021xUsergroupItem(TypedDict, total=False):
    """Nested item for user-group field."""
    name: str


class X8021xPayload(TypedDict, total=False):
    """Payload type for X8021x operations."""
    name: str
    security_mode: Literal["802.1X", "802.1X-mac-based"]
    user_group: str | list[str] | list[X8021xUsergroupItem]
    mac_auth_bypass: Literal["disable", "enable"]
    auth_order: Literal["dot1x-mab", "mab-dot1x", "mab"]
    auth_priority: Literal["legacy", "dot1x-mab", "mab-dot1x"]
    open_auth: Literal["disable", "enable"]
    eap_passthru: Literal["disable", "enable"]
    eap_auto_untagged_vlans: Literal["disable", "enable"]
    guest_vlan: Literal["disable", "enable"]
    guest_vlan_id: str
    guest_auth_delay: int
    auth_fail_vlan: Literal["disable", "enable"]
    auth_fail_vlan_id: str
    framevid_apply: Literal["disable", "enable"]
    radius_timeout_overwrite: Literal["disable", "enable"]
    policy_type: Literal["802.1X"]
    authserver_timeout_period: int
    authserver_timeout_vlan: Literal["disable", "enable"]
    authserver_timeout_vlanid: str
    authserver_timeout_tagged: Literal["disable", "lldp-voice", "static"]
    authserver_timeout_tagged_vlanid: str
    dacl: Literal["disable", "enable"]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class X8021xResponse(TypedDict, total=False):
    """Response type for X8021x - use with .dict property for typed dict access."""
    name: str
    security_mode: Literal["802.1X", "802.1X-mac-based"]
    user_group: list[X8021xUsergroupItem]
    mac_auth_bypass: Literal["disable", "enable"]
    auth_order: Literal["dot1x-mab", "mab-dot1x", "mab"]
    auth_priority: Literal["legacy", "dot1x-mab", "mab-dot1x"]
    open_auth: Literal["disable", "enable"]
    eap_passthru: Literal["disable", "enable"]
    eap_auto_untagged_vlans: Literal["disable", "enable"]
    guest_vlan: Literal["disable", "enable"]
    guest_vlan_id: str
    guest_auth_delay: int
    auth_fail_vlan: Literal["disable", "enable"]
    auth_fail_vlan_id: str
    framevid_apply: Literal["disable", "enable"]
    radius_timeout_overwrite: Literal["disable", "enable"]
    policy_type: Literal["802.1X"]
    authserver_timeout_period: int
    authserver_timeout_vlan: Literal["disable", "enable"]
    authserver_timeout_vlanid: str
    authserver_timeout_tagged: Literal["disable", "lldp-voice", "static"]
    authserver_timeout_tagged_vlanid: str
    dacl: Literal["disable", "enable"]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class X8021xUsergroupItemObject(FortiObject[X8021xUsergroupItem]):
    """Typed object for user-group table items with attribute access."""
    name: str


class X8021xObject(FortiObject):
    """Typed FortiObject for X8021x with field access."""
    name: str
    security_mode: Literal["802.1X", "802.1X-mac-based"]
    user_group: FortiObjectList[X8021xUsergroupItemObject]
    mac_auth_bypass: Literal["disable", "enable"]
    auth_order: Literal["dot1x-mab", "mab-dot1x", "mab"]
    auth_priority: Literal["legacy", "dot1x-mab", "mab-dot1x"]
    open_auth: Literal["disable", "enable"]
    eap_passthru: Literal["disable", "enable"]
    eap_auto_untagged_vlans: Literal["disable", "enable"]
    guest_vlan: Literal["disable", "enable"]
    guest_vlan_id: str
    guest_auth_delay: int
    auth_fail_vlan: Literal["disable", "enable"]
    auth_fail_vlan_id: str
    framevid_apply: Literal["disable", "enable"]
    radius_timeout_overwrite: Literal["disable", "enable"]
    policy_type: Literal["802.1X"]
    authserver_timeout_period: int
    authserver_timeout_vlan: Literal["disable", "enable"]
    authserver_timeout_vlanid: str
    authserver_timeout_tagged: Literal["disable", "lldp-voice", "static"]
    authserver_timeout_tagged_vlanid: str
    dacl: Literal["disable", "enable"]


# ================================================================
# Main Endpoint Class
# ================================================================

class X8021x:
    """
    
    Endpoint: switch_controller/security_policy/x802_1x
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> X8021xObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[X8021xObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: X8021xPayload | None = ...,
        name: str | None = ...,
        security_mode: Literal["802.1X", "802.1X-mac-based"] | None = ...,
        user_group: str | list[str] | list[X8021xUsergroupItem] | None = ...,
        mac_auth_bypass: Literal["disable", "enable"] | None = ...,
        auth_order: Literal["dot1x-mab", "mab-dot1x", "mab"] | None = ...,
        auth_priority: Literal["legacy", "dot1x-mab", "mab-dot1x"] | None = ...,
        open_auth: Literal["disable", "enable"] | None = ...,
        eap_passthru: Literal["disable", "enable"] | None = ...,
        eap_auto_untagged_vlans: Literal["disable", "enable"] | None = ...,
        guest_vlan: Literal["disable", "enable"] | None = ...,
        guest_vlan_id: str | None = ...,
        guest_auth_delay: int | None = ...,
        auth_fail_vlan: Literal["disable", "enable"] | None = ...,
        auth_fail_vlan_id: str | None = ...,
        framevid_apply: Literal["disable", "enable"] | None = ...,
        radius_timeout_overwrite: Literal["disable", "enable"] | None = ...,
        policy_type: Literal["802.1X"] | None = ...,
        authserver_timeout_period: int | None = ...,
        authserver_timeout_vlan: Literal["disable", "enable"] | None = ...,
        authserver_timeout_vlanid: str | None = ...,
        authserver_timeout_tagged: Literal["disable", "lldp-voice", "static"] | None = ...,
        authserver_timeout_tagged_vlanid: str | None = ...,
        dacl: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> X8021xObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: X8021xPayload | None = ...,
        name: str | None = ...,
        security_mode: Literal["802.1X", "802.1X-mac-based"] | None = ...,
        user_group: str | list[str] | list[X8021xUsergroupItem] | None = ...,
        mac_auth_bypass: Literal["disable", "enable"] | None = ...,
        auth_order: Literal["dot1x-mab", "mab-dot1x", "mab"] | None = ...,
        auth_priority: Literal["legacy", "dot1x-mab", "mab-dot1x"] | None = ...,
        open_auth: Literal["disable", "enable"] | None = ...,
        eap_passthru: Literal["disable", "enable"] | None = ...,
        eap_auto_untagged_vlans: Literal["disable", "enable"] | None = ...,
        guest_vlan: Literal["disable", "enable"] | None = ...,
        guest_vlan_id: str | None = ...,
        guest_auth_delay: int | None = ...,
        auth_fail_vlan: Literal["disable", "enable"] | None = ...,
        auth_fail_vlan_id: str | None = ...,
        framevid_apply: Literal["disable", "enable"] | None = ...,
        radius_timeout_overwrite: Literal["disable", "enable"] | None = ...,
        policy_type: Literal["802.1X"] | None = ...,
        authserver_timeout_period: int | None = ...,
        authserver_timeout_vlan: Literal["disable", "enable"] | None = ...,
        authserver_timeout_vlanid: str | None = ...,
        authserver_timeout_tagged: Literal["disable", "lldp-voice", "static"] | None = ...,
        authserver_timeout_tagged_vlanid: str | None = ...,
        dacl: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> X8021xObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: X8021xPayload | None = ...,
        name: str | None = ...,
        security_mode: Literal["802.1X", "802.1X-mac-based"] | None = ...,
        user_group: str | list[str] | list[X8021xUsergroupItem] | None = ...,
        mac_auth_bypass: Literal["disable", "enable"] | None = ...,
        auth_order: Literal["dot1x-mab", "mab-dot1x", "mab"] | None = ...,
        auth_priority: Literal["legacy", "dot1x-mab", "mab-dot1x"] | None = ...,
        open_auth: Literal["disable", "enable"] | None = ...,
        eap_passthru: Literal["disable", "enable"] | None = ...,
        eap_auto_untagged_vlans: Literal["disable", "enable"] | None = ...,
        guest_vlan: Literal["disable", "enable"] | None = ...,
        guest_vlan_id: str | None = ...,
        guest_auth_delay: int | None = ...,
        auth_fail_vlan: Literal["disable", "enable"] | None = ...,
        auth_fail_vlan_id: str | None = ...,
        framevid_apply: Literal["disable", "enable"] | None = ...,
        radius_timeout_overwrite: Literal["disable", "enable"] | None = ...,
        policy_type: Literal["802.1X"] | None = ...,
        authserver_timeout_period: int | None = ...,
        authserver_timeout_vlan: Literal["disable", "enable"] | None = ...,
        authserver_timeout_vlanid: str | None = ...,
        authserver_timeout_tagged: Literal["disable", "lldp-voice", "static"] | None = ...,
        authserver_timeout_tagged_vlanid: str | None = ...,
        dacl: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "X8021x",
    "X8021xPayload",
    "X8021xResponse",
    "X8021xObject",
]


"""FortiOS CMDB - Session6 category"""

from .close_multiple import CloseMultiple

__all__ = [
    "CloseMultiple",
    "Session6",
]


class Session6:
    """Session6 endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Session6 endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.close_multiple = CloseMultiple(client)



"""
Data formatting utilities for FortiOS objects and data structures.

Provides simple, type-agnostic conversion functions that handle any input gracefully.
Never raises exceptions - returns sensible defaults for edge cases.

"""

from __future__ import annotations

import json
from typing import Any

__all__ = [
    "to_json",
    "to_csv",
    "to_dict",
    "to_multiline",
    "to_list",
    "to_quoted",
    "to_table",
    "to_yaml",
    "to_xml",
    "to_key_value",
    "to_markdown_table",
    "to_dictlist",
    "to_listdict",
]


def to_json(data: Any, indent: int = 2, **kwargs: Any) -> str:
    """
    Convert any data to formatted JSON string.

    Handles objects with __dict__, converts sets/tuples to lists,
    and uses str() fallback for non-serializable types.

    Args:
        data: Any Python object to convert to JSON
        indent: Number of spaces for indentation (default: 2)
        **kwargs: Additional arguments passed to json.dumps()

    Returns:
        Formatted JSON string

    """

    def default_handler(obj: Any) -> Any:
        """Handle non-serializable objects."""
        if hasattr(obj, "__dict__"):
            return obj.__dict__
        if isinstance(obj, (set, tuple)):
            return list(obj)
        return str(obj)

    return json.dumps(data, indent=indent, default=default_handler, **kwargs)


def to_csv(data: Any, separator: str = ", ") -> str:
    """
    Convert any data to comma-separated string.

    Args:
        data: Any Python object to convert
        separator: String to use between items (default: ', ')

    Returns:
        Comma-separated string

    Examples:
        >>> to_csv(['port1', 'port2', 'port3'])
        'port1, port2, port3'

        >>> to_csv({'x': 1, 'y': 2, 'z': 3})
        'x=1, y=2, z=3'

        >>> to_csv('already a string')
        'already a string'

        >>> to_csv(None)
        ''

        >>> to_csv([1, 2, 3], separator=' | ')
        '1 | 2 | 3'

        >>> class Interface:
        ...     def __init__(self):
        ...         self.name = "port1"
        ...         self.ip = "10.0.0.1"
        >>> to_csv(Interface())
        'name=port1, ip=10.0.0.1'
    """
    if data is None:
        return ""

    if isinstance(data, str):
        return data

    if isinstance(data, (int, float, bool)):
        return str(data)

    if isinstance(data, (list, tuple, set)):
        return separator.join(str(item) for item in data)

    if isinstance(data, dict):
        return separator.join(f"{k}={v}" for k, v in data.items())

    if hasattr(data, "__dict__"):
        return to_csv(data.__dict__, separator)

    return str(data)


def to_dict(data: Any) -> dict[str, Any] | dict[int, Any]:
    """
    Convert any data to dictionary.

    Args:
        data: Any Python object to convert

    Returns:
        Dictionary representation of the data

    Examples:
        >>> class Policy:
        ...     def __init__(self):
        ...         self.name = "Allow-All"
        ...         self.action = "accept"
        >>> to_dict(Policy())
        {'name': 'Allow-All', 'action': 'accept'}

        >>> to_dict({'already': 'a dict'})
        {'already': 'a dict'}

        >>> to_dict([('a', 1), ('b', 2)])
        {'a': 1, 'b': 2}

        >>> to_dict(['x', 'y', 'z'])
        {0: 'x', 1: 'y', 2: 'z'}

        >>> to_dict('simple string')
        {'value': 'simple string'}

        >>> to_dict(None)
        {'value': None}
    """
    if isinstance(data, dict):
        return data

    if hasattr(data, "__dict__"):
        return data.__dict__

    if isinstance(data, (list, tuple)):
        # Check if it's a list of tuples (can be converted to dict)
        if data and all(
            isinstance(item, (list, tuple)) and len(item) == 2 for item in data
        ):
            try:
                return dict(data)
            except (TypeError, ValueError):
                pass
        # Otherwise convert to indexed dict
        return {i: v for i, v in enumerate(data)}

    # For primitives, wrap in value key
    return {"value": data}


def to_multiline(data: Any, separator: str = "\n") -> str:
    """
    Convert any data to newline-separated string.

    Args:
        data: Any Python object to convert
        separator: String to use between lines (default: '\\n')

    Returns:
        Newline-separated string

    Examples:
        >>> print(to_multiline(['port1', 'port2', 'port3']))
        port1
        port2
        port3

        >>> print(to_multiline({'name': 'policy1', 'action': 'accept'}))
        name: policy1
        action: accept

        >>> to_multiline('already a string')
        'already a string'

        >>> to_multiline(None)
        ''

        >>> class Policy:
        ...     def __init__(self):
        ...         self.name = "Allow-All"
        ...         self.policyid = 1
        >>> print(to_multiline(Policy()))
        name: Allow-All
        policyid: 1
    """
    if data is None:
        return ""

    if isinstance(data, str):
        return data

    if isinstance(data, (int, float, bool)):
        return str(data)

    if isinstance(data, (list, tuple, set)):
        return separator.join(str(item) for item in data)

    if isinstance(data, dict):
        return separator.join(f"{k}: {v}" for k, v in data.items())

    if hasattr(data, "__dict__"):
        return to_multiline(data.__dict__, separator)

    return str(data)


def to_list(data: Any, delimiter: str | None = None) -> list[Any]:
    """
    Convert any data to list.

    Args:
        data: Any Python object to convert
        delimiter: If data is a string, split by this delimiter.
                  If None and data is string with spaces, auto-splits by space.
                  Common delimiters: ',', ' ', '|', ';', etc.

    Returns:
        List representation of the data

    Examples:
        >>> to_list(['already', 'a', 'list'])
        ['already', 'a', 'list']

        >>> to_list(('tuple', 'to', 'list'))
        ['tuple', 'to', 'list']

        >>> to_list({'a', 'b', 'c'})  # set to list
        ['a', 'b', 'c']

        >>> to_list('port1,port2,port3', delimiter=',')
        ['port1', 'port2', 'port3']

        >>> to_list('port1 port2 port3')  # auto-splits on space
        ['port1', 'port2', 'port3']

        >>> to_list('80 443 8080')  # works with numbers as strings
        ['80', '443', '8080']

        >>> to_list('port1 | port2 | port3', delimiter=' | ')
        ['port1', 'port2', 'port3']

        >>> to_list('single_string')  # no spaces, returns as-is
        ['single_string']

        >>> to_list({'name': 'policy1', 'action': 'accept'})
        ['policy1', 'accept']

        >>> to_list(None)
        []

        >>> to_list(42)
        [42]

        >>> class Policy:
        ...     def __init__(self):
        ...         self.name = "Allow-All"
        ...         self.policyid = 1
        >>> to_list(Policy())
        ['Allow-All', 1]
    """
    if data is None:
        return []

    if isinstance(data, list):
        return data

    if isinstance(data, (tuple, set)):
        return list(data)

    if isinstance(data, str):
        if delimiter is not None:
            # Explicit delimiter provided
            return [item.strip() for item in data.split(delimiter)]
        elif " " in data:
            # Auto-split on space if no delimiter specified and string contains spaces
            return data.split()
        else:
            # Single item string with no spaces
            return [data]

    if isinstance(data, dict):
        return list(data.values())

    if hasattr(data, "__dict__"):
        return list(data.__dict__.values())

    # For primitives, return single-item list
    return [data]


def to_quoted(data: Any, quote: str = '"', separator: str = ", ") -> str:
    """
    Convert any data to quoted string representation.

    Args:
        data: Any Python object to convert
        quote: Quote character to use (default: '"')
        separator: String to use between quoted items (default: ', ')

    Returns:
        Quoted string representation

    Examples:
        >>> to_quoted(['port1', 'port2', 'port3'])
        '"port1", "port2", "port3"'

        >>> to_quoted({'x': 1, 'y': 2})
        '"x", "y"'

        >>> to_quoted('hello')
        '"hello"'

        >>> to_quoted(None)
        '""'

        >>> to_quoted([1, 2, 3], quote="'")
        "'1', '2', '3'"

        >>> class Interface:
        ...     def __init__(self):
        ...         self.name = "port1"
        ...         self.vlan = 10
        >>> to_quoted(Interface())
        '"name", "vlan"'
    """
    if data is None:
        return f"{quote}{quote}"

    if isinstance(data, str):
        return f"{quote}{data}{quote}"

    if isinstance(data, (int, float, bool)):
        return f"{quote}{data}{quote}"

    if isinstance(data, (list, tuple, set)):
        return separator.join(f"{quote}{item}{quote}" for item in data)

    if isinstance(data, dict):
        # Quote the keys
        return separator.join(f"{quote}{k}{quote}" for k in data.keys())

    if hasattr(data, "__dict__"):
        return to_quoted(data.__dict__, quote, separator)

    return f"{quote}{data}{quote}"


def to_table(data: Any, headers: bool = True, delimiter: str = " | ") -> str:
    """
    Convert data to table format.

    Args:
        data: Any Python object to convert (list of dicts, list of objects, etc.)
        headers: Whether to include headers (default: True)
        delimiter: Column delimiter (default: ' | ')

    Returns:
        Table-formatted string

    Examples:
        >>> policies = [
        ...     {'name': 'Allow-Web', 'action': 'accept', 'policyid': 1},
        ...     {'name': 'Block-All', 'action': 'deny', 'policyid': 2}
        ... ]
        >>> print(to_table(policies))
        name | action | policyid
        Allow-Web | accept | 1
        Block-All | deny | 2

        >>> to_table(policies, headers=False)
        'Allow-Web | accept | 1\\nBlock-All | deny | 2'

        >>> to_table(policies, delimiter=' || ')
        'name || action || policyid\\nAllow-Web || accept || 1\\nBlock-All || deny || 2'
    """
    if data is None:
        return ""

    if isinstance(data, str):
        return data

    # Convert single dict to list
    if isinstance(data, dict):
        data = [data]

    # Convert objects with __dict__ to dicts
    if not isinstance(data, (list, tuple)):
        if hasattr(data, "__dict__"):
            data = [data]
        else:
            return str(data)

    # Convert list of objects to list of dicts
    processed_data = []
    for item in data:
        if hasattr(item, "__dict__"):
            processed_data.append(item.__dict__)
        elif isinstance(item, dict):
            processed_data.append(item)
        else:
            processed_data.append({"value": item})

    if not processed_data:
        return ""

    # Get all unique keys
    all_keys = []
    for item in processed_data:
        for key in item.keys():
            if key not in all_keys:
                all_keys.append(key)

    lines = []

    # Add headers
    if headers:
        lines.append(delimiter.join(str(k) for k in all_keys))

    # Add rows
    for item in processed_data:
        row = delimiter.join(str(item.get(k, "")) for k in all_keys)
        lines.append(row)

    return "\n".join(lines)


def to_yaml(data: Any, indent: int = 2) -> str:
    """
    Convert data to YAML-like format (simple, no external dependencies).

    Args:
        data: Any Python object to convert
        indent: Number of spaces for indentation (default: 2)

    Returns:
        YAML-style string

    Examples:
        >>> policy = {'name': 'Allow-Web', 'action': 'accept', 'srcintf': ['port1', 'port2']}
        >>> print(to_yaml(policy))
        name: Allow-Web
        action: accept
        srcintf:
          - port1
          - port2

        >>> print(to_yaml({'nested': {'key': 'value'}}))
        nested:
          key: value
    """

    def _format_yaml(obj: Any, level: int = 0) -> str:
        indent_str = " " * (indent * level)

        if obj is None:
            return "null"

        if isinstance(obj, bool):
            return "true" if obj else "false"

        if isinstance(obj, (int, float)):
            return str(obj)

        if isinstance(obj, str):
            # Quote if contains special chars
            if ":" in obj or "#" in obj or obj.startswith((" ", "-")):
                return f"'{obj}'"
            return obj

        if isinstance(obj, (list, tuple)):
            if not obj:
                return "[]"
            lines = []
            for item in obj:
                if isinstance(item, (dict, list)):
                    # Complex item
                    lines.append(f"{indent_str}- ")
                    lines.append(_format_yaml(item, level + 1))
                else:
                    # Simple item
                    lines.append(f"{indent_str}- {_format_yaml(item, 0)}")
            return "\n".join(lines)

        if isinstance(obj, dict):
            if not obj:
                return "{}"
            lines = []
            for key, value in obj.items():
                if isinstance(value, dict):
                    lines.append(f"{indent_str}{key}:")
                    lines.append(_format_yaml(value, level + 1))
                elif isinstance(value, (list, tuple)):
                    lines.append(f"{indent_str}{key}:")
                    lines.append(_format_yaml(value, level + 1))
                else:
                    lines.append(
                        f"{indent_str}{key}: {_format_yaml(value, 0)}"
                    )
            return "\n".join(lines)

        if hasattr(obj, "__dict__"):
            return _format_yaml(obj.__dict__, level)

        return str(obj)

    return _format_yaml(data)


def to_xml(data: Any, root: str = "data", indent: int = 2) -> str:
    """
    Convert data to simple XML format (no external dependencies).

    Args:
        data: Any Python object to convert
        root: Root element name (default: 'data')
        indent: Number of spaces for indentation (default: 2)

    Returns:
        XML string

    Examples:
        >>> policy = {'name': 'Allow-Web', 'policyid': 1}
        >>> print(to_xml(policy, root='policy'))
        <policy>
          <name>Allow-Web</name>
          <policyid>1</policyid>
        </policy>

        >>> policies = [{'name': 'p1'}, {'name': 'p2'}]
        >>> print(to_xml(policies, root='policies'))
        <policies>
          <item>
            <name>p1</name>
          </item>
          <item>
            <name>p2</name>
          </item>
        </policies>
    """

    def _escape_xml(text: str) -> str:
        """Escape special XML characters."""
        # Order matters: & must be first to avoid double-escaping
        text = text.replace("&", "&amp;")
        text = text.replace("<", "&lt;")
        text = text.replace(">", "&gt;")
        text = text.replace('"', "&quot;")
        text = text.replace("'", "&apos;")
        return text

    def _format_xml(obj: Any, tag: str, level: int = 0) -> str:
        indent_str = " " * (indent * level)

        if obj is None:
            return f"{indent_str}<{tag}/>"

        if isinstance(obj, (str, int, float, bool)):
            value = _escape_xml(str(obj)) if isinstance(obj, str) else obj
            return f"{indent_str}<{tag}>{value}</{tag}>"

        if isinstance(obj, (list, tuple)):
            lines = [f"{indent_str}<{tag}>"]
            for item in obj:
                lines.append(_format_xml(item, "item", level + 1))
            lines.append(f"{indent_str}</{tag}>")
            return "\n".join(lines)

        if isinstance(obj, dict):
            lines = [f"{indent_str}<{tag}>"]
            for key, value in obj.items():
                # Sanitize key for XML
                safe_key = key.replace(" ", "_").replace("-", "_")
                lines.append(_format_xml(value, safe_key, level + 1))
            lines.append(f"{indent_str}</{tag}>")
            return "\n".join(lines)

        if hasattr(obj, "__dict__"):
            return _format_xml(obj.__dict__, tag, level)

        return f"{indent_str}<{tag}>{obj}</{tag}>"

    return _format_xml(data, root)


def to_key_value(
    data: Any, separator: str = "=", delimiter: str = "\n"
) -> str:
    """
    Convert data to key=value pairs format.

    Args:
        data: Any Python object to convert
        separator: Separator between key and value (default: '=')
        delimiter: Delimiter between pairs (default: '\\n')

    Returns:
        Key-value pairs string

    Examples:
        >>> config = {'host': '192.168.1.1', 'port': 443, 'verify': False}
        >>> print(to_key_value(config))
        host=192.168.1.1
        port=443
        verify=False

        >>> to_key_value(config, separator=': ', delimiter='; ')
        'host: 192.168.1.1; port: 443; verify: False'
    """
    if data is None:
        return ""

    if isinstance(data, str):
        return data

    if isinstance(data, dict):
        return delimiter.join(f"{k}{separator}{v}" for k, v in data.items())

    if hasattr(data, "__dict__"):
        return to_key_value(data.__dict__, separator, delimiter)

    if isinstance(data, (list, tuple)):
        # For lists, use index as key
        return delimiter.join(f"{i}{separator}{v}" for i, v in enumerate(data))

    return str(data)


def to_markdown_table(data: Any) -> str:
    """
    Convert data to Markdown table format.

    Args:
        data: List of dicts or list of objects

    Returns:
        Markdown table string

    Examples:
        >>> policies = [
        ...     {'name': 'Allow-Web', 'action': 'accept'},
        ...     {'name': 'Block-All', 'action': 'deny'}
        ... ]
        >>> print(to_markdown_table(policies))
        | name | action |
        | --- | --- |
        | Allow-Web | accept |
        | Block-All | deny |
    """
    if data is None:
        return ""

    if isinstance(data, str):
        return data

    # Convert single dict to list
    if isinstance(data, dict):
        data = [data]

    # Convert objects with __dict__ to dicts
    if not isinstance(data, (list, tuple)):
        if hasattr(data, "__dict__"):
            data = [data]
        else:
            return str(data)

    # Convert list of objects to list of dicts
    processed_data = []
    for item in data:
        if hasattr(item, "__dict__"):
            processed_data.append(item.__dict__)
        elif isinstance(item, dict):
            processed_data.append(item)
        else:
            processed_data.append({"value": item})

    if not processed_data:
        return ""

    # Get all unique keys
    all_keys = []
    for item in processed_data:
        for key in item.keys():
            if key not in all_keys:
                all_keys.append(key)

    lines = []

    # Add header
    lines.append("| " + " | ".join(str(k) for k in all_keys) + " |")
    lines.append("| " + " | ".join("---" for _ in all_keys) + " |")

    # Add rows
    for item in processed_data:
        row = "| " + " | ".join(str(item.get(k, "")) for k in all_keys) + " |"
        lines.append(row)

    return "\n".join(lines)


def to_dictlist(data: Any) -> list[dict[str, Any]]:
    """
    Convert dict of lists to list of dicts (columnar to row format).

    Useful for transforming columnar data into row-based records.

    Args:
        data: Dict where values are lists, or any convertible data

    Returns:
        List of dicts where each dict is one row

    Examples:
        >>> columnar = {'name': ['p1', 'p2'], 'action': ['accept', 'deny']}
        >>> to_dictlist(columnar)
        [{'name': 'p1', 'action': 'accept'}, {'name': 'p2', 'action': 'deny'}]

        >>> to_dictlist({'ports': ['80', '443', '8080']})
        [{'ports': '80'}, {'ports': '443'}, {'ports': '8080'}]

        >>> # Already list of dicts - returns as-is
        >>> to_dictlist([{'name': 'p1'}, {'name': 'p2'}])
        [{'name': 'p1'}, {'name': 'p2'}]

        >>> to_dictlist(None)
        []
    """
    if data is None:
        return []

    # Already list of dicts
    if isinstance(data, list):
        if all(isinstance(item, dict) for item in data):
            return data
        # List of objects with __dict__
        if all(hasattr(item, "__dict__") for item in data):
            return [item.__dict__ for item in data]
        # Convert simple list to list of value dicts
        return [{"value": item} for item in data]

    # Convert object to dict first
    if hasattr(data, "__dict__"):
        data = data.__dict__

    if not isinstance(data, dict):
        return [{"value": data}]

    # Dict of lists - convert to list of dicts
    # Get all keys
    keys = list(data.keys())
    if not keys:
        return []

    # Check if all values are lists/tuples
    values = [
        data[k] if isinstance(data[k], (list, tuple)) else [data[k]]
        for k in keys
    ]

    # Find max length
    max_len = max(len(v) for v in values) if values else 0

    # Pad lists to same length with None
    padded_values = []
    for v in values:
        if len(v) < max_len:
            padded_values.append(list(v) + [None] * (max_len - len(v)))
        else:
            padded_values.append(list(v))

    # Transpose to list of dicts
    result = []
    for i in range(max_len):
        row = {keys[j]: padded_values[j][i] for j in range(len(keys))}
        result.append(row)

    return result


def to_listdict(data: Any) -> dict[str, list[Any]]:
    """
    Convert list of dicts to dict of lists (row to columnar format).

    Useful for transforming row-based records into columnar data.

    Args:
        data: List of dicts, list of objects, or any convertible data

    Returns:
        Dict where keys are field names and values are lists

    Examples:
        >>> rows = [{'name': 'p1', 'action': 'accept'}, {'name': 'p2', 'action': 'deny'}]
        >>> to_listdict(rows)
        {'name': ['p1', 'p2'], 'action': ['accept', 'deny']}

        >>> to_listdict([{'ports': '80'}, {'ports': '443'}, {'ports': '8080'}])
        {'ports': ['80', '443', '8080']}

        >>> # Single dict becomes dict of single-item lists
        >>> to_listdict({'name': 'p1', 'action': 'accept'})
        {'name': ['p1'], 'action': ['accept']}

        >>> to_listdict(None)
        {}
    """
    if data is None:
        return {}

    # Already dict of lists
    if isinstance(data, dict):
        # Check if it's already dict of lists
        if all(isinstance(v, (list, tuple)) for v in data.values()):
            return {k: list(v) for k, v in data.items()}
        # Convert single values to lists
        return {k: [v] for k, v in data.items()}

    # Convert object to dict first
    if hasattr(data, "__dict__"):
        data = data.__dict__
        return {k: [v] for k, v in data.items()}

    if not isinstance(data, (list, tuple)):
        return {"value": [data]}

    # List of dicts - convert to dict of lists
    if not data:
        return {}

    # Convert objects to dicts
    processed_data = []
    for item in data:
        if hasattr(item, "__dict__"):
            processed_data.append(item.__dict__)
        elif isinstance(item, dict):
            processed_data.append(item)
        else:
            processed_data.append({"value": item})

    # Get all unique keys
    all_keys = []
    for item in processed_data:
        for key in item.keys():
            if key not in all_keys:
                all_keys.append(key)

    # Build dict of lists
    result = {key: [] for key in all_keys}
    for item in processed_data:
        for key in all_keys:
            result[key].append(item.get(key, None))

    return result



"""Type stubs for SNMP category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .community import Community
    from .mib_view import MibView
    from .rmon_stat import RmonStat
    from .sysinfo import Sysinfo
    from .user import User

__all__ = [
    "Community",
    "MibView",
    "RmonStat",
    "Sysinfo",
    "User",
    "Snmp",
]


class Snmp:
    """SNMP API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    community: Community
    mib_view: MibView
    rmon_stat: RmonStat
    sysinfo: Sysinfo
    user: User

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize snmp category with HTTP client."""
        ...



"""FortiOS CMDB - Acl6 category (stub)"""

from typing import Any
from ..acl6_base import Acl6 as Acl6Base
from .clear_counters import ClearCounters

class Acl6(Acl6Base):
    """Acl6 endpoints wrapper for CMDB API."""

    clear_counters: ClearCounters

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: voip/profile
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class ProfileSipDict(TypedDict, total=False):
    """Nested object type for sip field."""
    status: Literal["disable", "enable"]
    rtp: Literal["disable", "enable"]
    nat_port_range: str
    open_register_pinhole: Literal["disable", "enable"]
    open_contact_pinhole: Literal["disable", "enable"]
    strict_register: Literal["disable", "enable"]
    register_rate: int
    register_rate_track: Literal["none", "src-ip", "dest-ip"]
    invite_rate: int
    invite_rate_track: Literal["none", "src-ip", "dest-ip"]
    max_dialogs: int
    max_line_length: int
    block_long_lines: Literal["disable", "enable"]
    block_unknown: Literal["disable", "enable"]
    call_keepalive: int
    block_ack: Literal["disable", "enable"]
    block_bye: Literal["disable", "enable"]
    block_cancel: Literal["disable", "enable"]
    block_info: Literal["disable", "enable"]
    block_invite: Literal["disable", "enable"]
    block_message: Literal["disable", "enable"]
    block_notify: Literal["disable", "enable"]
    block_options: Literal["disable", "enable"]
    block_prack: Literal["disable", "enable"]
    block_publish: Literal["disable", "enable"]
    block_refer: Literal["disable", "enable"]
    block_register: Literal["disable", "enable"]
    block_subscribe: Literal["disable", "enable"]
    block_update: Literal["disable", "enable"]
    register_contact_trace: Literal["disable", "enable"]
    open_via_pinhole: Literal["disable", "enable"]
    open_record_route_pinhole: Literal["disable", "enable"]
    rfc2543_branch: Literal["disable", "enable"]
    log_violations: Literal["disable", "enable"]
    log_call_summary: Literal["disable", "enable"]
    nat_trace: Literal["disable", "enable"]
    subscribe_rate: int
    subscribe_rate_track: Literal["none", "src-ip", "dest-ip"]
    message_rate: int
    message_rate_track: Literal["none", "src-ip", "dest-ip"]
    notify_rate: int
    notify_rate_track: Literal["none", "src-ip", "dest-ip"]
    refer_rate: int
    refer_rate_track: Literal["none", "src-ip", "dest-ip"]
    update_rate: int
    update_rate_track: Literal["none", "src-ip", "dest-ip"]
    options_rate: int
    options_rate_track: Literal["none", "src-ip", "dest-ip"]
    ack_rate: int
    ack_rate_track: Literal["none", "src-ip", "dest-ip"]
    prack_rate: int
    prack_rate_track: Literal["none", "src-ip", "dest-ip"]
    info_rate: int
    info_rate_track: Literal["none", "src-ip", "dest-ip"]
    publish_rate: int
    publish_rate_track: Literal["none", "src-ip", "dest-ip"]
    bye_rate: int
    bye_rate_track: Literal["none", "src-ip", "dest-ip"]
    cancel_rate: int
    cancel_rate_track: Literal["none", "src-ip", "dest-ip"]
    preserve_override: Literal["disable", "enable"]
    no_sdp_fixup: Literal["disable", "enable"]
    contact_fixup: Literal["disable", "enable"]
    max_idle_dialogs: int
    block_geo_red_options: Literal["disable", "enable"]
    hosted_nat_traversal: Literal["disable", "enable"]
    hnt_restrict_source_ip: Literal["disable", "enable"]
    call_id_regex: str
    content_type_regex: str
    max_body_length: int
    unknown_header: Literal["discard", "pass", "respond"]
    malformed_request_line: Literal["discard", "pass", "respond"]
    malformed_header_via: Literal["discard", "pass", "respond"]
    malformed_header_from: Literal["discard", "pass", "respond"]
    malformed_header_to: Literal["discard", "pass", "respond"]
    malformed_header_call_id: Literal["discard", "pass", "respond"]
    malformed_header_cseq: Literal["discard", "pass", "respond"]
    malformed_header_rack: Literal["discard", "pass", "respond"]
    malformed_header_rseq: Literal["discard", "pass", "respond"]
    malformed_header_contact: Literal["discard", "pass", "respond"]
    malformed_header_record_route: Literal["discard", "pass", "respond"]
    malformed_header_route: Literal["discard", "pass", "respond"]
    malformed_header_expires: Literal["discard", "pass", "respond"]
    malformed_header_content_type: Literal["discard", "pass", "respond"]
    malformed_header_content_length: Literal["discard", "pass", "respond"]
    malformed_header_max_forwards: Literal["discard", "pass", "respond"]
    malformed_header_allow: Literal["discard", "pass", "respond"]
    malformed_header_p_asserted_identity: Literal["discard", "pass", "respond"]
    malformed_header_no_require: Literal["discard", "pass", "respond"]
    malformed_header_no_proxy_require: Literal["discard", "pass", "respond"]
    malformed_header_sdp_v: Literal["discard", "pass", "respond"]
    malformed_header_sdp_o: Literal["discard", "pass", "respond"]
    malformed_header_sdp_s: Literal["discard", "pass", "respond"]
    malformed_header_sdp_i: Literal["discard", "pass", "respond"]
    malformed_header_sdp_c: Literal["discard", "pass", "respond"]
    malformed_header_sdp_b: Literal["discard", "pass", "respond"]
    malformed_header_sdp_z: Literal["discard", "pass", "respond"]
    malformed_header_sdp_k: Literal["discard", "pass", "respond"]
    malformed_header_sdp_a: Literal["discard", "pass", "respond"]
    malformed_header_sdp_t: Literal["discard", "pass", "respond"]
    malformed_header_sdp_r: Literal["discard", "pass", "respond"]
    malformed_header_sdp_m: Literal["discard", "pass", "respond"]
    provisional_invite_expiry_time: int
    ips_rtp: Literal["disable", "enable"]
    ssl_mode: Literal["off", "full"]
    ssl_send_empty_frags: Literal["enable", "disable"]
    ssl_client_renegotiation: Literal["allow", "deny", "secure"]
    ssl_algorithm: Literal["high", "medium", "low"]
    ssl_pfs: Literal["require", "deny", "allow"]
    ssl_min_version: Literal["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"]
    ssl_max_version: Literal["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"]
    ssl_client_certificate: str
    ssl_server_certificate: str
    ssl_auth_client: str
    ssl_auth_server: str


class ProfileSccpDict(TypedDict, total=False):
    """Nested object type for sccp field."""
    status: Literal["disable", "enable"]
    block_mcast: Literal["disable", "enable"]
    verify_header: Literal["disable", "enable"]
    log_call_summary: Literal["disable", "enable"]
    log_violations: Literal["disable", "enable"]
    max_calls: int


class ProfileMsrpDict(TypedDict, total=False):
    """Nested object type for msrp field."""
    status: Literal["disable", "enable"]
    log_violations: Literal["disable", "enable"]
    max_msg_size: int
    max_msg_size_action: Literal["pass", "block", "reset", "monitor"]


class ProfilePayload(TypedDict, total=False):
    """Payload type for Profile operations."""
    name: str
    feature_set: Literal["ips", "voipd"]
    comment: str
    sip: ProfileSipDict
    sccp: ProfileSccpDict
    msrp: ProfileMsrpDict


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class ProfileResponse(TypedDict, total=False):
    """Response type for Profile - use with .dict property for typed dict access."""
    name: str
    feature_set: Literal["ips", "voipd"]
    comment: str
    sip: ProfileSipDict
    sccp: ProfileSccpDict
    msrp: ProfileMsrpDict


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class ProfileSipObject(FortiObject):
    """Nested object for sip field with attribute access."""
    status: Literal["disable", "enable"]
    rtp: Literal["disable", "enable"]
    nat_port_range: str
    open_register_pinhole: Literal["disable", "enable"]
    open_contact_pinhole: Literal["disable", "enable"]
    strict_register: Literal["disable", "enable"]
    register_rate: int
    register_rate_track: Literal["none", "src-ip", "dest-ip"]
    invite_rate: int
    invite_rate_track: Literal["none", "src-ip", "dest-ip"]
    max_dialogs: int
    max_line_length: int
    block_long_lines: Literal["disable", "enable"]
    block_unknown: Literal["disable", "enable"]
    call_keepalive: int
    block_ack: Literal["disable", "enable"]
    block_bye: Literal["disable", "enable"]
    block_cancel: Literal["disable", "enable"]
    block_info: Literal["disable", "enable"]
    block_invite: Literal["disable", "enable"]
    block_message: Literal["disable", "enable"]
    block_notify: Literal["disable", "enable"]
    block_options: Literal["disable", "enable"]
    block_prack: Literal["disable", "enable"]
    block_publish: Literal["disable", "enable"]
    block_refer: Literal["disable", "enable"]
    block_register: Literal["disable", "enable"]
    block_subscribe: Literal["disable", "enable"]
    block_update: Literal["disable", "enable"]
    register_contact_trace: Literal["disable", "enable"]
    open_via_pinhole: Literal["disable", "enable"]
    open_record_route_pinhole: Literal["disable", "enable"]
    rfc2543_branch: Literal["disable", "enable"]
    log_violations: Literal["disable", "enable"]
    log_call_summary: Literal["disable", "enable"]
    nat_trace: Literal["disable", "enable"]
    subscribe_rate: int
    subscribe_rate_track: Literal["none", "src-ip", "dest-ip"]
    message_rate: int
    message_rate_track: Literal["none", "src-ip", "dest-ip"]
    notify_rate: int
    notify_rate_track: Literal["none", "src-ip", "dest-ip"]
    refer_rate: int
    refer_rate_track: Literal["none", "src-ip", "dest-ip"]
    update_rate: int
    update_rate_track: Literal["none", "src-ip", "dest-ip"]
    options_rate: int
    options_rate_track: Literal["none", "src-ip", "dest-ip"]
    ack_rate: int
    ack_rate_track: Literal["none", "src-ip", "dest-ip"]
    prack_rate: int
    prack_rate_track: Literal["none", "src-ip", "dest-ip"]
    info_rate: int
    info_rate_track: Literal["none", "src-ip", "dest-ip"]
    publish_rate: int
    publish_rate_track: Literal["none", "src-ip", "dest-ip"]
    bye_rate: int
    bye_rate_track: Literal["none", "src-ip", "dest-ip"]
    cancel_rate: int
    cancel_rate_track: Literal["none", "src-ip", "dest-ip"]
    preserve_override: Literal["disable", "enable"]
    no_sdp_fixup: Literal["disable", "enable"]
    contact_fixup: Literal["disable", "enable"]
    max_idle_dialogs: int
    block_geo_red_options: Literal["disable", "enable"]
    hosted_nat_traversal: Literal["disable", "enable"]
    hnt_restrict_source_ip: Literal["disable", "enable"]
    call_id_regex: str
    content_type_regex: str
    max_body_length: int
    unknown_header: Literal["discard", "pass", "respond"]
    malformed_request_line: Literal["discard", "pass", "respond"]
    malformed_header_via: Literal["discard", "pass", "respond"]
    malformed_header_from: Literal["discard", "pass", "respond"]
    malformed_header_to: Literal["discard", "pass", "respond"]
    malformed_header_call_id: Literal["discard", "pass", "respond"]
    malformed_header_cseq: Literal["discard", "pass", "respond"]
    malformed_header_rack: Literal["discard", "pass", "respond"]
    malformed_header_rseq: Literal["discard", "pass", "respond"]
    malformed_header_contact: Literal["discard", "pass", "respond"]
    malformed_header_record_route: Literal["discard", "pass", "respond"]
    malformed_header_route: Literal["discard", "pass", "respond"]
    malformed_header_expires: Literal["discard", "pass", "respond"]
    malformed_header_content_type: Literal["discard", "pass", "respond"]
    malformed_header_content_length: Literal["discard", "pass", "respond"]
    malformed_header_max_forwards: Literal["discard", "pass", "respond"]
    malformed_header_allow: Literal["discard", "pass", "respond"]
    malformed_header_p_asserted_identity: Literal["discard", "pass", "respond"]
    malformed_header_no_require: Literal["discard", "pass", "respond"]
    malformed_header_no_proxy_require: Literal["discard", "pass", "respond"]
    malformed_header_sdp_v: Literal["discard", "pass", "respond"]
    malformed_header_sdp_o: Literal["discard", "pass", "respond"]
    malformed_header_sdp_s: Literal["discard", "pass", "respond"]
    malformed_header_sdp_i: Literal["discard", "pass", "respond"]
    malformed_header_sdp_c: Literal["discard", "pass", "respond"]
    malformed_header_sdp_b: Literal["discard", "pass", "respond"]
    malformed_header_sdp_z: Literal["discard", "pass", "respond"]
    malformed_header_sdp_k: Literal["discard", "pass", "respond"]
    malformed_header_sdp_a: Literal["discard", "pass", "respond"]
    malformed_header_sdp_t: Literal["discard", "pass", "respond"]
    malformed_header_sdp_r: Literal["discard", "pass", "respond"]
    malformed_header_sdp_m: Literal["discard", "pass", "respond"]
    provisional_invite_expiry_time: int
    ips_rtp: Literal["disable", "enable"]
    ssl_mode: Literal["off", "full"]
    ssl_send_empty_frags: Literal["enable", "disable"]
    ssl_client_renegotiation: Literal["allow", "deny", "secure"]
    ssl_algorithm: Literal["high", "medium", "low"]
    ssl_pfs: Literal["require", "deny", "allow"]
    ssl_min_version: Literal["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"]
    ssl_max_version: Literal["ssl-3.0", "tls-1.0", "tls-1.1", "tls-1.2", "tls-1.3"]
    ssl_client_certificate: str
    ssl_server_certificate: str
    ssl_auth_client: str
    ssl_auth_server: str


class ProfileSccpObject(FortiObject):
    """Nested object for sccp field with attribute access."""
    status: Literal["disable", "enable"]
    block_mcast: Literal["disable", "enable"]
    verify_header: Literal["disable", "enable"]
    log_call_summary: Literal["disable", "enable"]
    log_violations: Literal["disable", "enable"]
    max_calls: int


class ProfileMsrpObject(FortiObject):
    """Nested object for msrp field with attribute access."""
    status: Literal["disable", "enable"]
    log_violations: Literal["disable", "enable"]
    max_msg_size: int
    max_msg_size_action: Literal["pass", "block", "reset", "monitor"]


class ProfileObject(FortiObject):
    """Typed FortiObject for Profile with field access."""
    name: str
    feature_set: Literal["ips", "voipd"]
    comment: str
    sip: ProfileSipObject
    sccp: ProfileSccpObject
    msrp: ProfileMsrpObject


# ================================================================
# Main Endpoint Class
# ================================================================

class Profile:
    """
    
    Endpoint: voip/profile
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[ProfileObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: ProfilePayload | None = ...,
        name: str | None = ...,
        feature_set: Literal["ips", "voipd"] | None = ...,
        comment: str | None = ...,
        sip: ProfileSipDict | None = ...,
        sccp: ProfileSccpDict | None = ...,
        msrp: ProfileMsrpDict | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: ProfilePayload | None = ...,
        name: str | None = ...,
        feature_set: Literal["ips", "voipd"] | None = ...,
        comment: str | None = ...,
        sip: ProfileSipDict | None = ...,
        sccp: ProfileSccpDict | None = ...,
        msrp: ProfileMsrpDict | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> ProfileObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: ProfilePayload | None = ...,
        name: str | None = ...,
        feature_set: Literal["ips", "voipd"] | None = ...,
        comment: str | None = ...,
        sip: ProfileSipDict | None = ...,
        sccp: ProfileSccpDict | None = ...,
        msrp: ProfileMsrpDict | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Profile",
    "ProfilePayload",
    "ProfileResponse",
    "ProfileObject",
]


"""FortiOS CMDB - Local category"""

from .change_password import ChangePassword

__all__ = [
    "ChangePassword",
    "Local",
]


class Local:
    """Local endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Local endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.change_password = ChangePassword(client)



from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_SSH_KEX_ALGO: Literal["diffie-hellman-group1-sha1", "diffie-hellman-group14-sha1", "diffie-hellman-group14-sha256", "diffie-hellman-group16-sha512", "diffie-hellman-group18-sha512", "diffie-hellman-group-exchange-sha1", "diffie-hellman-group-exchange-sha256", "curve25519-sha256@libssh.org", "ecdh-sha2-nistp256", "ecdh-sha2-nistp384", "ecdh-sha2-nistp521"]
VALID_BODY_SSH_ENC_ALGO: Literal["chacha20-poly1305@openssh.com", "aes128-ctr", "aes192-ctr", "aes256-ctr", "arcfour256", "arcfour128", "aes128-cbc", "3des-cbc", "blowfish-cbc", "cast128-cbc", "aes192-cbc", "aes256-cbc", "arcfour", "rijndael-cbc@lysator.liu.se", "aes128-gcm@openssh.com", "aes256-gcm@openssh.com"]
VALID_BODY_SSH_MAC_ALGO: Literal["hmac-md5", "hmac-md5-etm@openssh.com", "hmac-md5-96", "hmac-md5-96-etm@openssh.com", "hmac-sha1", "hmac-sha1-etm@openssh.com", "hmac-sha2-256", "hmac-sha2-256-etm@openssh.com", "hmac-sha2-512", "hmac-sha2-512-etm@openssh.com", "hmac-ripemd160", "hmac-ripemd160@openssh.com", "hmac-ripemd160-etm@openssh.com", "umac-64@openssh.com", "umac-128@openssh.com", "umac-64-etm@openssh.com", "umac-128-etm@openssh.com"]
VALID_BODY_SSH_HSK_ALGO: Literal["ssh-rsa", "ecdsa-sha2-nistp521", "ecdsa-sha2-nistp384", "ecdsa-sha2-nistp256", "rsa-sha2-256", "rsa-sha2-512", "ssh-ed25519"]
VALID_BODY_SSH_HSK_OVERRIDE: Literal["disable", "enable"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]
DEPRECATED_FIELDS: dict[str, dict[str, str]]
REQUIRED_FIELDS: list[str]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_SSH_KEX_ALGO",
    "VALID_BODY_SSH_ENC_ALGO",
    "VALID_BODY_SSH_MAC_ALGO",
    "VALID_BODY_SSH_HSK_ALGO",
    "VALID_BODY_SSH_HSK_OVERRIDE",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "DEPRECATED_FIELDS",
    "REQUIRED_FIELDS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]


""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: router/ospf6
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class Ospf6AreaIpseckeysItem(TypedDict, total=False):
    """Nested item for area.ipsec-keys field."""
    spi: int
    auth_key: str
    enc_key: str


class Ospf6AreaRangeItem(TypedDict, total=False):
    """Nested item for area.range field."""
    id: int
    prefix6: str
    advertise: Literal["disable", "enable"]


class Ospf6AreaVirtuallinkItem(TypedDict, total=False):
    """Nested item for area.virtual-link field."""
    name: str
    dead_interval: int
    hello_interval: int
    retransmit_interval: int
    transmit_delay: int
    peer: str
    authentication: Literal["none", "ah", "esp", "area"]
    key_rollover_interval: int
    ipsec_auth_alg: Literal["md5", "sha1", "sha256", "sha384", "sha512"]
    ipsec_enc_alg: Literal["null", "des", "3des", "aes128", "aes192", "aes256"]
    ipsec_keys: str | list[str]


class Ospf6Ospf6interfaceIpseckeysItem(TypedDict, total=False):
    """Nested item for ospf6-interface.ipsec-keys field."""
    spi: int
    auth_key: str
    enc_key: str


class Ospf6Ospf6interfaceNeighborItem(TypedDict, total=False):
    """Nested item for ospf6-interface.neighbor field."""
    ip6: str
    poll_interval: int
    cost: int
    priority: int


class Ospf6AreaItem(TypedDict, total=False):
    """Nested item for area field."""
    id: str
    default_cost: int
    nssa_translator_role: Literal["candidate", "never", "always"]
    stub_type: Literal["no-summary", "summary"]
    type: Literal["regular", "nssa", "stub"]
    nssa_default_information_originate: Literal["enable", "disable"]
    nssa_default_information_originate_metric: int
    nssa_default_information_originate_metric_type: Literal["1", "2"]
    nssa_redistribution: Literal["enable", "disable"]
    authentication: Literal["none", "ah", "esp"]
    key_rollover_interval: int
    ipsec_auth_alg: Literal["md5", "sha1", "sha256", "sha384", "sha512"]
    ipsec_enc_alg: Literal["null", "des", "3des", "aes128", "aes192", "aes256"]
    ipsec_keys: str | list[str] | list[Ospf6AreaIpseckeysItem]
    range: str | list[str] | list[Ospf6AreaRangeItem]
    virtual_link: str | list[str] | list[Ospf6AreaVirtuallinkItem]


class Ospf6Ospf6interfaceItem(TypedDict, total=False):
    """Nested item for ospf6-interface field."""
    name: str
    area_id: str
    interface: str
    retransmit_interval: int
    transmit_delay: int
    cost: int
    priority: int
    dead_interval: int
    hello_interval: int
    status: Literal["disable", "enable"]
    network_type: Literal["broadcast", "point-to-point", "non-broadcast", "point-to-multipoint", "point-to-multipoint-non-broadcast"]
    bfd: Literal["global", "enable", "disable"]
    mtu: int
    mtu_ignore: Literal["enable", "disable"]
    authentication: Literal["none", "ah", "esp", "area"]
    key_rollover_interval: int
    ipsec_auth_alg: Literal["md5", "sha1", "sha256", "sha384", "sha512"]
    ipsec_enc_alg: Literal["null", "des", "3des", "aes128", "aes192", "aes256"]
    ipsec_keys: str | list[str] | list[Ospf6Ospf6interfaceIpseckeysItem]
    neighbor: str | list[str] | list[Ospf6Ospf6interfaceNeighborItem]


class Ospf6RedistributeItem(TypedDict, total=False):
    """Nested item for redistribute field."""
    name: str
    status: Literal["enable", "disable"]
    metric: int
    routemap: str
    metric_type: Literal["1", "2"]


class Ospf6PassiveinterfaceItem(TypedDict, total=False):
    """Nested item for passive-interface field."""
    name: str


class Ospf6SummaryaddressItem(TypedDict, total=False):
    """Nested item for summary-address field."""
    id: int
    prefix6: str
    advertise: Literal["disable", "enable"]
    tag: int


class Ospf6Payload(TypedDict, total=False):
    """Payload type for Ospf6 operations."""
    abr_type: Literal["cisco", "ibm", "standard"]
    auto_cost_ref_bandwidth: int
    default_information_originate: Literal["enable", "always", "disable"]
    log_neighbour_changes: Literal["enable", "disable"]
    default_information_metric: int
    default_information_metric_type: Literal["1", "2"]
    default_information_route_map: str
    default_metric: int
    router_id: str
    spf_timers: str
    bfd: Literal["enable", "disable"]
    restart_mode: Literal["none", "graceful-restart"]
    restart_period: int
    restart_on_topology_change: Literal["enable", "disable"]
    area: str | list[str] | list[Ospf6AreaItem]
    ospf6_interface: str | list[str] | list[Ospf6Ospf6interfaceItem]
    redistribute: str | list[str] | list[Ospf6RedistributeItem]
    passive_interface: str | list[str] | list[Ospf6PassiveinterfaceItem]
    summary_address: str | list[str] | list[Ospf6SummaryaddressItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class Ospf6Response(TypedDict, total=False):
    """Response type for Ospf6 - use with .dict property for typed dict access."""
    abr_type: Literal["cisco", "ibm", "standard"]
    auto_cost_ref_bandwidth: int
    default_information_originate: Literal["enable", "always", "disable"]
    log_neighbour_changes: Literal["enable", "disable"]
    default_information_metric: int
    default_information_metric_type: Literal["1", "2"]
    default_information_route_map: str
    default_metric: int
    router_id: str
    spf_timers: str
    bfd: Literal["enable", "disable"]
    restart_mode: Literal["none", "graceful-restart"]
    restart_period: int
    restart_on_topology_change: Literal["enable", "disable"]
    area: list[Ospf6AreaItem]
    ospf6_interface: list[Ospf6Ospf6interfaceItem]
    redistribute: list[Ospf6RedistributeItem]
    passive_interface: list[Ospf6PassiveinterfaceItem]
    summary_address: list[Ospf6SummaryaddressItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class Ospf6AreaIpseckeysItemObject(FortiObject[Ospf6AreaIpseckeysItem]):
    """Typed object for area.ipsec-keys table items with attribute access."""
    spi: int
    auth_key: str
    enc_key: str


class Ospf6AreaRangeItemObject(FortiObject[Ospf6AreaRangeItem]):
    """Typed object for area.range table items with attribute access."""
    id: int
    prefix6: str
    advertise: Literal["disable", "enable"]


class Ospf6AreaVirtuallinkItemObject(FortiObject[Ospf6AreaVirtuallinkItem]):
    """Typed object for area.virtual-link table items with attribute access."""
    name: str
    dead_interval: int
    hello_interval: int
    retransmit_interval: int
    transmit_delay: int
    peer: str
    authentication: Literal["none", "ah", "esp", "area"]
    key_rollover_interval: int
    ipsec_auth_alg: Literal["md5", "sha1", "sha256", "sha384", "sha512"]
    ipsec_enc_alg: Literal["null", "des", "3des", "aes128", "aes192", "aes256"]
    ipsec_keys: str | list[str]


class Ospf6Ospf6interfaceIpseckeysItemObject(FortiObject[Ospf6Ospf6interfaceIpseckeysItem]):
    """Typed object for ospf6-interface.ipsec-keys table items with attribute access."""
    spi: int
    auth_key: str
    enc_key: str


class Ospf6Ospf6interfaceNeighborItemObject(FortiObject[Ospf6Ospf6interfaceNeighborItem]):
    """Typed object for ospf6-interface.neighbor table items with attribute access."""
    ip6: str
    poll_interval: int
    cost: int
    priority: int


class Ospf6AreaItemObject(FortiObject[Ospf6AreaItem]):
    """Typed object for area table items with attribute access."""
    id: str
    default_cost: int
    nssa_translator_role: Literal["candidate", "never", "always"]
    stub_type: Literal["no-summary", "summary"]
    type: Literal["regular", "nssa", "stub"]
    nssa_default_information_originate: Literal["enable", "disable"]
    nssa_default_information_originate_metric: int
    nssa_default_information_originate_metric_type: Literal["1", "2"]
    nssa_redistribution: Literal["enable", "disable"]
    authentication: Literal["none", "ah", "esp"]
    key_rollover_interval: int
    ipsec_auth_alg: Literal["md5", "sha1", "sha256", "sha384", "sha512"]
    ipsec_enc_alg: Literal["null", "des", "3des", "aes128", "aes192", "aes256"]
    ipsec_keys: FortiObjectList[Ospf6AreaIpseckeysItemObject]
    range: FortiObjectList[Ospf6AreaRangeItemObject]
    virtual_link: FortiObjectList[Ospf6AreaVirtuallinkItemObject]


class Ospf6Ospf6interfaceItemObject(FortiObject[Ospf6Ospf6interfaceItem]):
    """Typed object for ospf6-interface table items with attribute access."""
    name: str
    area_id: str
    interface: str
    retransmit_interval: int
    transmit_delay: int
    cost: int
    priority: int
    dead_interval: int
    hello_interval: int
    status: Literal["disable", "enable"]
    network_type: Literal["broadcast", "point-to-point", "non-broadcast", "point-to-multipoint", "point-to-multipoint-non-broadcast"]
    bfd: Literal["global", "enable", "disable"]
    mtu: int
    mtu_ignore: Literal["enable", "disable"]
    authentication: Literal["none", "ah", "esp", "area"]
    key_rollover_interval: int
    ipsec_auth_alg: Literal["md5", "sha1", "sha256", "sha384", "sha512"]
    ipsec_enc_alg: Literal["null", "des", "3des", "aes128", "aes192", "aes256"]
    ipsec_keys: FortiObjectList[Ospf6Ospf6interfaceIpseckeysItemObject]
    neighbor: FortiObjectList[Ospf6Ospf6interfaceNeighborItemObject]


class Ospf6RedistributeItemObject(FortiObject[Ospf6RedistributeItem]):
    """Typed object for redistribute table items with attribute access."""
    name: str
    status: Literal["enable", "disable"]
    metric: int
    routemap: str
    metric_type: Literal["1", "2"]


class Ospf6PassiveinterfaceItemObject(FortiObject[Ospf6PassiveinterfaceItem]):
    """Typed object for passive-interface table items with attribute access."""
    name: str


class Ospf6SummaryaddressItemObject(FortiObject[Ospf6SummaryaddressItem]):
    """Typed object for summary-address table items with attribute access."""
    id: int
    prefix6: str
    advertise: Literal["disable", "enable"]
    tag: int


class Ospf6Object(FortiObject):
    """Typed FortiObject for Ospf6 with field access."""
    abr_type: Literal["cisco", "ibm", "standard"]
    auto_cost_ref_bandwidth: int
    default_information_originate: Literal["enable", "always", "disable"]
    log_neighbour_changes: Literal["enable", "disable"]
    default_information_metric: int
    default_information_metric_type: Literal["1", "2"]
    default_information_route_map: str
    default_metric: int
    router_id: str
    spf_timers: str
    bfd: Literal["enable", "disable"]
    restart_mode: Literal["none", "graceful-restart"]
    restart_period: int
    restart_on_topology_change: Literal["enable", "disable"]
    area: FortiObjectList[Ospf6AreaItemObject]
    ospf6_interface: FortiObjectList[Ospf6Ospf6interfaceItemObject]
    redistribute: FortiObjectList[Ospf6RedistributeItemObject]
    passive_interface: FortiObjectList[Ospf6PassiveinterfaceItemObject]
    summary_address: FortiObjectList[Ospf6SummaryaddressItemObject]


# ================================================================
# Main Endpoint Class
# ================================================================

class Ospf6:
    """
    
    Endpoint: router/ospf6
    Category: cmdb
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # Singleton endpoint (no mkey)
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> Ospf6Object: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...


    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: Ospf6Payload | None = ...,
        abr_type: Literal["cisco", "ibm", "standard"] | None = ...,
        auto_cost_ref_bandwidth: int | None = ...,
        default_information_originate: Literal["enable", "always", "disable"] | None = ...,
        log_neighbour_changes: Literal["enable", "disable"] | None = ...,
        default_information_metric: int | None = ...,
        default_information_metric_type: Literal["1", "2"] | None = ...,
        default_information_route_map: str | None = ...,
        default_metric: int | None = ...,
        router_id: str | None = ...,
        spf_timers: str | None = ...,
        bfd: Literal["enable", "disable"] | None = ...,
        restart_mode: Literal["none", "graceful-restart"] | None = ...,
        restart_period: int | None = ...,
        restart_on_topology_change: Literal["enable", "disable"] | None = ...,
        area: str | list[str] | list[Ospf6AreaItem] | None = ...,
        ospf6_interface: str | list[str] | list[Ospf6Ospf6interfaceItem] | None = ...,
        redistribute: str | list[str] | list[Ospf6RedistributeItem] | None = ...,
        passive_interface: str | list[str] | list[Ospf6PassiveinterfaceItem] | None = ...,
        summary_address: str | list[str] | list[Ospf6SummaryaddressItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> Ospf6Object: ...


    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: Ospf6Payload | None = ...,
        abr_type: Literal["cisco", "ibm", "standard"] | None = ...,
        auto_cost_ref_bandwidth: int | None = ...,
        default_information_originate: Literal["enable", "always", "disable"] | None = ...,
        log_neighbour_changes: Literal["enable", "disable"] | None = ...,
        default_information_metric: int | None = ...,
        default_information_metric_type: Literal["1", "2"] | None = ...,
        default_information_route_map: str | None = ...,
        default_metric: int | None = ...,
        router_id: str | None = ...,
        spf_timers: str | None = ...,
        bfd: Literal["enable", "disable"] | None = ...,
        restart_mode: Literal["none", "graceful-restart"] | None = ...,
        restart_period: int | None = ...,
        restart_on_topology_change: Literal["enable", "disable"] | None = ...,
        area: str | list[str] | list[Ospf6AreaItem] | None = ...,
        ospf6_interface: str | list[str] | list[Ospf6Ospf6interfaceItem] | None = ...,
        redistribute: str | list[str] | list[Ospf6RedistributeItem] | None = ...,
        passive_interface: str | list[str] | list[Ospf6PassiveinterfaceItem] | None = ...,
        summary_address: str | list[str] | list[Ospf6SummaryaddressItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Ospf6",
    "Ospf6Payload",
    "Ospf6Response",
    "Ospf6Object",
]


"""Type stubs for LOG category."""

from typing import TYPE_CHECKING

from .disk import Disk as Disk
from .fortianalyzer import Fortianalyzer as Fortianalyzer
from .forticloud import Forticloud as Forticloud
from .memory import Memory as Memory
from .search import Search as Search

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient


class Log:
    """Container for LOG endpoints.
    
    Provides access to log query endpoints for different storage locations:
    - disk: Logs stored on local disk
    - memory: Logs stored in memory
    - fortianalyzer: Logs from FortiAnalyzer
    - forticloud: Logs from FortiCloud
    - search: Log search operations
    """
    disk: Disk
    fortianalyzer: Fortianalyzer
    forticloud: Forticloud
    memory: Memory
    search: Search

    def __init__(self, client: IHTTPClient) -> None: ...


"""FortiOS CMDB - Ippool category (stub)"""

from typing import Any
from ..ippool_base import Ippool as IppoolBase
from .mapping import Mapping

class Ippool(IppoolBase):
    """Ippool endpoints wrapper for CMDB API."""

    mapping: Mapping

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...



"""Validation helpers for vpn/certificate/setting - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "interface",  # Specify outgoing interface to reach server.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "ocsp-status": "disable",
    "ocsp-option": "server",
    "proxy": "",
    "proxy-port": 8080,
    "proxy-username": "",
    "source-ip": "",
    "ocsp-default-server": "",
    "interface-select-method": "auto",
    "interface": "",
    "vrf-select": 0,
    "check-ca-cert": "enable",
    "check-ca-chain": "disable",
    "subject-match": "substring",
    "subject-set": "subset",
    "cn-match": "substring",
    "cn-allow-multi": "enable",
    "strict-ocsp-check": "disable",
    "ssl-min-proto-version": "default",
    "cmp-save-extra-certs": "disable",
    "cmp-key-usage-checking": "enable",
    "cert-expire-warning": 14,
    "certname-rsa1024": "Fortinet_SSL_RSA1024",
    "certname-rsa2048": "Fortinet_SSL_RSA2048",
    "certname-rsa4096": "Fortinet_SSL_RSA4096",
    "certname-dsa1024": "Fortinet_SSL_DSA1024",
    "certname-dsa2048": "Fortinet_SSL_DSA2048",
    "certname-ecdsa256": "Fortinet_SSL_ECDSA256",
    "certname-ecdsa384": "Fortinet_SSL_ECDSA384",
    "certname-ecdsa521": "Fortinet_SSL_ECDSA521",
    "certname-ed25519": "Fortinet_SSL_ED25519",
    "certname-ed448": "Fortinet_SSL_ED448",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "ocsp-status": "option",  # Enable/disable receiving certificates using the OCSP.
    "ocsp-option": "option",  # Specify whether the OCSP URL is from certificate or configur
    "proxy": "string",  # Proxy server FQDN or IP for OCSP/CA queries during certifica
    "proxy-port": "integer",  # Proxy server port (1 - 65535, default = 8080).
    "proxy-username": "string",  # Proxy server user name.
    "proxy-password": "password",  # Proxy server password.
    "source-ip": "string",  # Source IP address for dynamic AIA and OCSP queries.
    "ocsp-default-server": "string",  # Default OCSP server.
    "interface-select-method": "option",  # Specify how to select outgoing interface to reach server.
    "interface": "string",  # Specify outgoing interface to reach server.
    "vrf-select": "integer",  # VRF ID used for connection to server.
    "check-ca-cert": "option",  # Enable/disable verification of the user certificate and pass
    "check-ca-chain": "option",  # Enable/disable verification of the entire certificate chain 
    "subject-match": "option",  # When searching for a matching certificate, control how to do
    "subject-set": "option",  # When searching for a matching certificate, control how to do
    "cn-match": "option",  # When searching for a matching certificate, control how to do
    "cn-allow-multi": "option",  # When searching for a matching certificate, allow multiple CN
    "crl-verification": "string",  # CRL verification options.
    "strict-ocsp-check": "option",  # Enable/disable strict mode OCSP checking.
    "ssl-min-proto-version": "option",  # Minimum supported protocol version for SSL/TLS connections (
    "cmp-save-extra-certs": "option",  # Enable/disable saving extra certificates in CMP mode (defaul
    "cmp-key-usage-checking": "option",  # Enable/disable server certificate key usage checking in CMP 
    "cert-expire-warning": "integer",  # Number of days before a certificate expires to send a warnin
    "certname-rsa1024": "string",  # 1024 bit RSA key certificate for re-signing server certifica
    "certname-rsa2048": "string",  # 2048 bit RSA key certificate for re-signing server certifica
    "certname-rsa4096": "string",  # 4096 bit RSA key certificate for re-signing server certifica
    "certname-dsa1024": "string",  # 1024 bit DSA key certificate for re-signing server certifica
    "certname-dsa2048": "string",  # 2048 bit DSA key certificate for re-signing server certifica
    "certname-ecdsa256": "string",  # 256 bit ECDSA key certificate for re-signing server certific
    "certname-ecdsa384": "string",  # 384 bit ECDSA key certificate for re-signing server certific
    "certname-ecdsa521": "string",  # 521 bit ECDSA key certificate for re-signing server certific
    "certname-ed25519": "string",  # 253 bit EdDSA key certificate for re-signing server certific
    "certname-ed448": "string",  # 456 bit EdDSA key certificate for re-signing server certific
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "ocsp-status": "Enable/disable receiving certificates using the OCSP.",
    "ocsp-option": "Specify whether the OCSP URL is from certificate or configured OCSP server.",
    "proxy": "Proxy server FQDN or IP for OCSP/CA queries during certificate verification.",
    "proxy-port": "Proxy server port (1 - 65535, default = 8080).",
    "proxy-username": "Proxy server user name.",
    "proxy-password": "Proxy server password.",
    "source-ip": "Source IP address for dynamic AIA and OCSP queries.",
    "ocsp-default-server": "Default OCSP server.",
    "interface-select-method": "Specify how to select outgoing interface to reach server.",
    "interface": "Specify outgoing interface to reach server.",
    "vrf-select": "VRF ID used for connection to server.",
    "check-ca-cert": "Enable/disable verification of the user certificate and pass authentication if any CA in the chain is trusted (default = enable).",
    "check-ca-chain": "Enable/disable verification of the entire certificate chain and pass authentication only if the chain is complete and all of the CAs in the chain are trusted (default = disable).",
    "subject-match": "When searching for a matching certificate, control how to do RDN value matching with certificate subject name (default = substring).",
    "subject-set": "When searching for a matching certificate, control how to do RDN set matching with certificate subject name (default = subset).",
    "cn-match": "When searching for a matching certificate, control how to do CN value matching with certificate subject name (default = substring).",
    "cn-allow-multi": "When searching for a matching certificate, allow multiple CN fields in certificate subject name (default = enable).",
    "crl-verification": "CRL verification options.",
    "strict-ocsp-check": "Enable/disable strict mode OCSP checking.",
    "ssl-min-proto-version": "Minimum supported protocol version for SSL/TLS connections (default is to follow system global setting).",
    "cmp-save-extra-certs": "Enable/disable saving extra certificates in CMP mode (default = disable).",
    "cmp-key-usage-checking": "Enable/disable server certificate key usage checking in CMP mode (default = enable).",
    "cert-expire-warning": "Number of days before a certificate expires to send a warning. Set to 0 to disable sending of the warning (0 - 100, default = 14).",
    "certname-rsa1024": "1024 bit RSA key certificate for re-signing server certificates for SSL inspection.",
    "certname-rsa2048": "2048 bit RSA key certificate for re-signing server certificates for SSL inspection.",
    "certname-rsa4096": "4096 bit RSA key certificate for re-signing server certificates for SSL inspection.",
    "certname-dsa1024": "1024 bit DSA key certificate for re-signing server certificates for SSL inspection.",
    "certname-dsa2048": "2048 bit DSA key certificate for re-signing server certificates for SSL inspection.",
    "certname-ecdsa256": "256 bit ECDSA key certificate for re-signing server certificates for SSL inspection.",
    "certname-ecdsa384": "384 bit ECDSA key certificate for re-signing server certificates for SSL inspection.",
    "certname-ecdsa521": "521 bit ECDSA key certificate for re-signing server certificates for SSL inspection.",
    "certname-ed25519": "253 bit EdDSA key certificate for re-signing server certificates for SSL inspection.",
    "certname-ed448": "456 bit EdDSA key certificate for re-signing server certificates for SSL inspection.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "proxy": {"type": "string", "max_length": 127},
    "proxy-port": {"type": "integer", "min": 1, "max": 65535},
    "proxy-username": {"type": "string", "max_length": 63},
    "source-ip": {"type": "string", "max_length": 63},
    "ocsp-default-server": {"type": "string", "max_length": 35},
    "interface": {"type": "string", "max_length": 15},
    "vrf-select": {"type": "integer", "min": 0, "max": 511},
    "cert-expire-warning": {"type": "integer", "min": 0, "max": 100},
    "certname-rsa1024": {"type": "string", "max_length": 35},
    "certname-rsa2048": {"type": "string", "max_length": 35},
    "certname-rsa4096": {"type": "string", "max_length": 35},
    "certname-dsa1024": {"type": "string", "max_length": 35},
    "certname-dsa2048": {"type": "string", "max_length": 35},
    "certname-ecdsa256": {"type": "string", "max_length": 35},
    "certname-ecdsa384": {"type": "string", "max_length": 35},
    "certname-ecdsa521": {"type": "string", "max_length": 35},
    "certname-ed25519": {"type": "string", "max_length": 35},
    "certname-ed448": {"type": "string", "max_length": 35},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "crl-verification": {
        "expiry": {
            "type": "option",
            "help": "CRL verification option when CRL is expired (default = ignore).",
            "default": "ignore",
            "options": ["ignore", "revoke"],
        },
        "leaf-crl-absence": {
            "type": "option",
            "help": "CRL verification option when leaf CRL is absent (default = ignore).",
            "default": "ignore",
            "options": ["ignore", "revoke"],
        },
        "chain-crl-absence": {
            "type": "option",
            "help": "CRL verification option when CRL of any certificate in chain is absent (default = ignore).",
            "default": "ignore",
            "options": ["ignore", "revoke"],
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_OCSP_STATUS = [
    "enable",
    "mandatory",
    "disable",
]
VALID_BODY_OCSP_OPTION = [
    "certificate",
    "server",
]
VALID_BODY_INTERFACE_SELECT_METHOD = [
    "auto",
    "sdwan",
    "specify",
]
VALID_BODY_CHECK_CA_CERT = [
    "enable",
    "disable",
]
VALID_BODY_CHECK_CA_CHAIN = [
    "enable",
    "disable",
]
VALID_BODY_SUBJECT_MATCH = [
    "substring",
    "value",
]
VALID_BODY_SUBJECT_SET = [
    "subset",
    "superset",
]
VALID_BODY_CN_MATCH = [
    "substring",
    "value",
]
VALID_BODY_CN_ALLOW_MULTI = [
    "disable",
    "enable",
]
VALID_BODY_STRICT_OCSP_CHECK = [
    "enable",
    "disable",
]
VALID_BODY_SSL_MIN_PROTO_VERSION = [
    "default",
    "SSLv3",
    "TLSv1",
    "TLSv1-1",
    "TLSv1-2",
    "TLSv1-3",
]
VALID_BODY_CMP_SAVE_EXTRA_CERTS = [
    "enable",
    "disable",
]
VALID_BODY_CMP_KEY_USAGE_CHECKING = [
    "enable",
    "disable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_vpn_certificate_setting_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for vpn/certificate/setting."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_vpn_certificate_setting_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new vpn/certificate/setting object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "ocsp-status" in payload:
        is_valid, error = _validate_enum_field(
            "ocsp-status",
            payload["ocsp-status"],
            VALID_BODY_OCSP_STATUS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ocsp-option" in payload:
        is_valid, error = _validate_enum_field(
            "ocsp-option",
            payload["ocsp-option"],
            VALID_BODY_OCSP_OPTION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "interface-select-method" in payload:
        is_valid, error = _validate_enum_field(
            "interface-select-method",
            payload["interface-select-method"],
            VALID_BODY_INTERFACE_SELECT_METHOD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "check-ca-cert" in payload:
        is_valid, error = _validate_enum_field(
            "check-ca-cert",
            payload["check-ca-cert"],
            VALID_BODY_CHECK_CA_CERT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "check-ca-chain" in payload:
        is_valid, error = _validate_enum_field(
            "check-ca-chain",
            payload["check-ca-chain"],
            VALID_BODY_CHECK_CA_CHAIN,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "subject-match" in payload:
        is_valid, error = _validate_enum_field(
            "subject-match",
            payload["subject-match"],
            VALID_BODY_SUBJECT_MATCH,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "subject-set" in payload:
        is_valid, error = _validate_enum_field(
            "subject-set",
            payload["subject-set"],
            VALID_BODY_SUBJECT_SET,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "cn-match" in payload:
        is_valid, error = _validate_enum_field(
            "cn-match",
            payload["cn-match"],
            VALID_BODY_CN_MATCH,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "cn-allow-multi" in payload:
        is_valid, error = _validate_enum_field(
            "cn-allow-multi",
            payload["cn-allow-multi"],
            VALID_BODY_CN_ALLOW_MULTI,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "strict-ocsp-check" in payload:
        is_valid, error = _validate_enum_field(
            "strict-ocsp-check",
            payload["strict-ocsp-check"],
            VALID_BODY_STRICT_OCSP_CHECK,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-min-proto-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-min-proto-version",
            payload["ssl-min-proto-version"],
            VALID_BODY_SSL_MIN_PROTO_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "cmp-save-extra-certs" in payload:
        is_valid, error = _validate_enum_field(
            "cmp-save-extra-certs",
            payload["cmp-save-extra-certs"],
            VALID_BODY_CMP_SAVE_EXTRA_CERTS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "cmp-key-usage-checking" in payload:
        is_valid, error = _validate_enum_field(
            "cmp-key-usage-checking",
            payload["cmp-key-usage-checking"],
            VALID_BODY_CMP_KEY_USAGE_CHECKING,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_vpn_certificate_setting_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update vpn/certificate/setting."""
    # Validate enum values using central function
    if "ocsp-status" in payload:
        is_valid, error = _validate_enum_field(
            "ocsp-status",
            payload["ocsp-status"],
            VALID_BODY_OCSP_STATUS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ocsp-option" in payload:
        is_valid, error = _validate_enum_field(
            "ocsp-option",
            payload["ocsp-option"],
            VALID_BODY_OCSP_OPTION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "interface-select-method" in payload:
        is_valid, error = _validate_enum_field(
            "interface-select-method",
            payload["interface-select-method"],
            VALID_BODY_INTERFACE_SELECT_METHOD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "check-ca-cert" in payload:
        is_valid, error = _validate_enum_field(
            "check-ca-cert",
            payload["check-ca-cert"],
            VALID_BODY_CHECK_CA_CERT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "check-ca-chain" in payload:
        is_valid, error = _validate_enum_field(
            "check-ca-chain",
            payload["check-ca-chain"],
            VALID_BODY_CHECK_CA_CHAIN,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "subject-match" in payload:
        is_valid, error = _validate_enum_field(
            "subject-match",
            payload["subject-match"],
            VALID_BODY_SUBJECT_MATCH,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "subject-set" in payload:
        is_valid, error = _validate_enum_field(
            "subject-set",
            payload["subject-set"],
            VALID_BODY_SUBJECT_SET,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "cn-match" in payload:
        is_valid, error = _validate_enum_field(
            "cn-match",
            payload["cn-match"],
            VALID_BODY_CN_MATCH,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "cn-allow-multi" in payload:
        is_valid, error = _validate_enum_field(
            "cn-allow-multi",
            payload["cn-allow-multi"],
            VALID_BODY_CN_ALLOW_MULTI,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "strict-ocsp-check" in payload:
        is_valid, error = _validate_enum_field(
            "strict-ocsp-check",
            payload["strict-ocsp-check"],
            VALID_BODY_STRICT_OCSP_CHECK,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ssl-min-proto-version" in payload:
        is_valid, error = _validate_enum_field(
            "ssl-min-proto-version",
            payload["ssl-min-proto-version"],
            VALID_BODY_SSL_MIN_PROTO_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "cmp-save-extra-certs" in payload:
        is_valid, error = _validate_enum_field(
            "cmp-save-extra-certs",
            payload["cmp-save-extra-certs"],
            VALID_BODY_CMP_SAVE_EXTRA_CERTS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "cmp-key-usage-checking" in payload:
        is_valid, error = _validate_enum_field(
            "cmp-key-usage-checking",
            payload["cmp-key-usage-checking"],
            VALID_BODY_CMP_KEY_USAGE_CHECKING,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "vpn/certificate/setting",
    "category": "cmdb",
    "api_path": "vpn.certificate/setting",
    "help": "VPN certificate setting.",
    "total_fields": 33,
    "required_fields_count": 1,
    "fields_with_defaults_count": 31,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""Validation helpers for system/vdom_sflow - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "vdom-sflow": "disable",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "vdom-sflow": "option",  # Enable/disable the sFlow configuration for the current VDOM.
    "collectors": "string",  # sFlow collectors.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "vdom-sflow": "Enable/disable the sFlow configuration for the current VDOM.",
    "collectors": "sFlow collectors.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "collectors": {
        "id": {
            "type": "integer",
            "help": "ID.",
            "required": True,
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "collector-ip": {
            "type": "ipv4-address",
            "help": "IP addresses of the sFlow collectors that sFlow agents added to interfaces in this VDOM send sFlow datagrams to.",
            "required": True,
            "default": "0.0.0.0",
        },
        "collector-port": {
            "type": "integer",
            "help": "UDP port number used for sending sFlow datagrams (configure only if required by your sFlow collector or your network configuration) (0 - 65535, default = 6343).",
            "default": 6343,
            "min_value": 0,
            "max_value": 65535,
        },
        "source-ip": {
            "type": "ipv4-address",
            "help": "Source IP address for sFlow agent.",
            "default": "0.0.0.0",
        },
        "interface-select-method": {
            "type": "option",
            "help": "Specify how to select outgoing interface to reach server.",
            "default": "auto",
            "options": ["auto", "sdwan", "specify"],
        },
        "interface": {
            "type": "string",
            "help": "Specify outgoing interface to reach server.",
            "required": True,
            "default": "",
            "max_length": 15,
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_VDOM_SFLOW = [
    "enable",
    "disable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_system_vdom_sflow_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for system/vdom_sflow."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_system_vdom_sflow_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new system/vdom_sflow object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "vdom-sflow" in payload:
        is_valid, error = _validate_enum_field(
            "vdom-sflow",
            payload["vdom-sflow"],
            VALID_BODY_VDOM_SFLOW,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_system_vdom_sflow_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update system/vdom_sflow."""
    # Validate enum values using central function
    if "vdom-sflow" in payload:
        is_valid, error = _validate_enum_field(
            "vdom-sflow",
            payload["vdom-sflow"],
            VALID_BODY_VDOM_SFLOW,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "system/vdom_sflow",
    "category": "cmdb",
    "api_path": "system/vdom-sflow",
    "help": "Configure sFlow per VDOM to add or change the IP address and UDP port that FortiGate sFlow agents in this VDOM use to send sFlow datagrams to an sFlow collector.",
    "total_fields": 2,
    "required_fields_count": 0,
    "fields_with_defaults_count": 1,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""FortiOS CMDB - Firmware category (stub)"""

from typing import Any
from ..firmware_base import Firmware as FirmwareBase
from .download import Download
from .push import Push
from .upload import Upload

class Firmware(FirmwareBase):
    """Firmware endpoints wrapper for CMDB API."""

    download: Download
    push: Push
    upload: Upload

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...



"""FortiOS CMDB - Replacemsg category"""

from .admin import Admin
from .alertmail import Alertmail
from .auth import Auth
from .automation import Automation
from .fortiguard_wf import FortiguardWf
from .http import Http
from .mail import Mail
from .nac_quar import NacQuar
from .spam import Spam
from .sslvpn import Sslvpn
from .traffic_quota import TrafficQuota
from .utm import Utm

__all__ = [
    "Admin",
    "Alertmail",
    "Auth",
    "Automation",
    "FortiguardWf",
    "Http",
    "Mail",
    "NacQuar",
    "Replacemsg",
    "Spam",
    "Sslvpn",
    "TrafficQuota",
    "Utm",
]


class Replacemsg:
    """Replacemsg endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Replacemsg endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.admin = Admin(client)
        self.alertmail = Alertmail(client)
        self.auth = Auth(client)
        self.automation = Automation(client)
        self.fortiguard_wf = FortiguardWf(client)
        self.http = Http(client)
        self.mail = Mail(client)
        self.nac_quar = NacQuar(client)
        self.spam = Spam(client)
        self.sslvpn = Sslvpn(client)
        self.traffic_quota = TrafficQuota(client)
        self.utm = Utm(client)



"""FortiOS CMDB - ConfigRevision category"""

from ..config_revision_base import ConfigRevision as ConfigRevisionBase
from .delete import Delete
from .file import File
from .info import Info
from .save import Save
from .update_comments import UpdateComments

__all__ = [
    "ConfigRevision",
    "Delete",
    "File",
    "Info",
    "Save",
    "UpdateComments",
]


class ConfigRevision(ConfigRevisionBase):
    """ConfigRevision endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """ConfigRevision endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        super().__init__(client)  # Initialize base class with GET methods
        self.delete = Delete(client)
        self.file = File(client)
        self.info = Info(client)
        self.save = Save(client)
        self.update_comments = UpdateComments(client)



"""FortiOS CMDB - Acl category"""

from .group import Group
from .ingress import Ingress

__all__ = [
    "Acl",
    "Group",
    "Ingress",
]


class Acl:
    """Acl endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Acl endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.group = Group(client)
        self.ingress = Ingress(client)



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: system/accprofile
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class AccprofileNetgrppermissionDict(TypedDict, total=False):
    """Nested object type for netgrp-permission field."""
    cfg: Literal["none", "read", "read-write"]
    packet_capture: Literal["none", "read", "read-write"]
    route_cfg: Literal["none", "read", "read-write"]


class AccprofileSysgrppermissionDict(TypedDict, total=False):
    """Nested object type for sysgrp-permission field."""
    admin: Literal["none", "read", "read-write"]
    upd: Literal["none", "read", "read-write"]
    cfg: Literal["none", "read", "read-write"]
    mnt: Literal["none", "read", "read-write"]


class AccprofileFwgrppermissionDict(TypedDict, total=False):
    """Nested object type for fwgrp-permission field."""
    policy: Literal["none", "read", "read-write"]
    address: Literal["none", "read", "read-write"]
    service: Literal["none", "read", "read-write"]
    schedule: Literal["none", "read", "read-write"]
    others: Literal["none", "read", "read-write"]


class AccprofileLoggrppermissionDict(TypedDict, total=False):
    """Nested object type for loggrp-permission field."""
    config: Literal["none", "read", "read-write"]
    data_access: Literal["none", "read", "read-write"]
    report_access: Literal["none", "read", "read-write"]
    threat_weight: Literal["none", "read", "read-write"]


class AccprofileUtmgrppermissionDict(TypedDict, total=False):
    """Nested object type for utmgrp-permission field."""
    antivirus: Literal["none", "read", "read-write"]
    ips: Literal["none", "read", "read-write"]
    webfilter: Literal["none", "read", "read-write"]
    emailfilter: Literal["none", "read", "read-write"]
    dlp: Literal["none", "read", "read-write"]
    file_filter: Literal["none", "read", "read-write"]
    application_control: Literal["none", "read", "read-write"]
    icap: Literal["none", "read", "read-write"]
    voip: Literal["none", "read", "read-write"]
    waf: Literal["none", "read", "read-write"]
    dnsfilter: Literal["none", "read", "read-write"]
    endpoint_control: Literal["none", "read", "read-write"]
    videofilter: Literal["none", "read", "read-write"]
    virtual_patch: Literal["none", "read", "read-write"]
    casb: Literal["none", "read", "read-write"]
    telemetry: Literal["none", "read", "read-write"]


class AccprofileSecfabgrppermissionDict(TypedDict, total=False):
    """Nested object type for secfabgrp-permission field."""
    csfsys: Literal["none", "read", "read-write"]
    csffoo: Literal["none", "read", "read-write"]


class AccprofilePayload(TypedDict, total=False):
    """Payload type for Accprofile operations."""
    name: str
    scope: Literal["vdom", "global"]
    comments: str
    secfabgrp: Literal["none", "read", "read-write", "custom"]
    ftviewgrp: Literal["none", "read", "read-write"]
    authgrp: Literal["none", "read", "read-write"]
    sysgrp: Literal["none", "read", "read-write", "custom"]
    netgrp: Literal["none", "read", "read-write", "custom"]
    loggrp: Literal["none", "read", "read-write", "custom"]
    fwgrp: Literal["none", "read", "read-write", "custom"]
    vpngrp: Literal["none", "read", "read-write"]
    utmgrp: Literal["none", "read", "read-write", "custom"]
    wanoptgrp: Literal["none", "read", "read-write"]
    wifi: Literal["none", "read", "read-write"]
    netgrp_permission: AccprofileNetgrppermissionDict
    sysgrp_permission: AccprofileSysgrppermissionDict
    fwgrp_permission: AccprofileFwgrppermissionDict
    loggrp_permission: AccprofileLoggrppermissionDict
    utmgrp_permission: AccprofileUtmgrppermissionDict
    secfabgrp_permission: AccprofileSecfabgrppermissionDict
    admintimeout_override: Literal["enable", "disable"]
    admintimeout: int
    cli_diagnose: Literal["enable", "disable"]
    cli_get: Literal["enable", "disable"]
    cli_show: Literal["enable", "disable"]
    cli_exec: Literal["enable", "disable"]
    cli_config: Literal["enable", "disable"]
    system_execute_ssh: Literal["enable", "disable"]
    system_execute_telnet: Literal["enable", "disable"]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class AccprofileResponse(TypedDict, total=False):
    """Response type for Accprofile - use with .dict property for typed dict access."""
    name: str
    scope: Literal["vdom", "global"]
    comments: str
    secfabgrp: Literal["none", "read", "read-write", "custom"]
    ftviewgrp: Literal["none", "read", "read-write"]
    authgrp: Literal["none", "read", "read-write"]
    sysgrp: Literal["none", "read", "read-write", "custom"]
    netgrp: Literal["none", "read", "read-write", "custom"]
    loggrp: Literal["none", "read", "read-write", "custom"]
    fwgrp: Literal["none", "read", "read-write", "custom"]
    vpngrp: Literal["none", "read", "read-write"]
    utmgrp: Literal["none", "read", "read-write", "custom"]
    wanoptgrp: Literal["none", "read", "read-write"]
    wifi: Literal["none", "read", "read-write"]
    netgrp_permission: AccprofileNetgrppermissionDict
    sysgrp_permission: AccprofileSysgrppermissionDict
    fwgrp_permission: AccprofileFwgrppermissionDict
    loggrp_permission: AccprofileLoggrppermissionDict
    utmgrp_permission: AccprofileUtmgrppermissionDict
    secfabgrp_permission: AccprofileSecfabgrppermissionDict
    admintimeout_override: Literal["enable", "disable"]
    admintimeout: int
    cli_diagnose: Literal["enable", "disable"]
    cli_get: Literal["enable", "disable"]
    cli_show: Literal["enable", "disable"]
    cli_exec: Literal["enable", "disable"]
    cli_config: Literal["enable", "disable"]
    system_execute_ssh: Literal["enable", "disable"]
    system_execute_telnet: Literal["enable", "disable"]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class AccprofileNetgrppermissionObject(FortiObject):
    """Nested object for netgrp-permission field with attribute access."""
    cfg: Literal["none", "read", "read-write"]
    packet_capture: Literal["none", "read", "read-write"]
    route_cfg: Literal["none", "read", "read-write"]


class AccprofileSysgrppermissionObject(FortiObject):
    """Nested object for sysgrp-permission field with attribute access."""
    admin: Literal["none", "read", "read-write"]
    upd: Literal["none", "read", "read-write"]
    cfg: Literal["none", "read", "read-write"]
    mnt: Literal["none", "read", "read-write"]


class AccprofileFwgrppermissionObject(FortiObject):
    """Nested object for fwgrp-permission field with attribute access."""
    policy: Literal["none", "read", "read-write"]
    address: Literal["none", "read", "read-write"]
    service: Literal["none", "read", "read-write"]
    schedule: Literal["none", "read", "read-write"]
    others: Literal["none", "read", "read-write"]


class AccprofileLoggrppermissionObject(FortiObject):
    """Nested object for loggrp-permission field with attribute access."""
    config: Literal["none", "read", "read-write"]
    data_access: Literal["none", "read", "read-write"]
    report_access: Literal["none", "read", "read-write"]
    threat_weight: Literal["none", "read", "read-write"]


class AccprofileUtmgrppermissionObject(FortiObject):
    """Nested object for utmgrp-permission field with attribute access."""
    antivirus: Literal["none", "read", "read-write"]
    ips: Literal["none", "read", "read-write"]
    webfilter: Literal["none", "read", "read-write"]
    emailfilter: Literal["none", "read", "read-write"]
    dlp: Literal["none", "read", "read-write"]
    file_filter: Literal["none", "read", "read-write"]
    application_control: Literal["none", "read", "read-write"]
    icap: Literal["none", "read", "read-write"]
    voip: Literal["none", "read", "read-write"]
    waf: Literal["none", "read", "read-write"]
    dnsfilter: Literal["none", "read", "read-write"]
    endpoint_control: Literal["none", "read", "read-write"]
    videofilter: Literal["none", "read", "read-write"]
    virtual_patch: Literal["none", "read", "read-write"]
    casb: Literal["none", "read", "read-write"]
    telemetry: Literal["none", "read", "read-write"]


class AccprofileSecfabgrppermissionObject(FortiObject):
    """Nested object for secfabgrp-permission field with attribute access."""
    csfsys: Literal["none", "read", "read-write"]
    csffoo: Literal["none", "read", "read-write"]


class AccprofileObject(FortiObject):
    """Typed FortiObject for Accprofile with field access."""
    name: str
    scope: Literal["vdom", "global"]
    comments: str
    secfabgrp: Literal["none", "read", "read-write", "custom"]
    ftviewgrp: Literal["none", "read", "read-write"]
    authgrp: Literal["none", "read", "read-write"]
    sysgrp: Literal["none", "read", "read-write", "custom"]
    netgrp: Literal["none", "read", "read-write", "custom"]
    loggrp: Literal["none", "read", "read-write", "custom"]
    fwgrp: Literal["none", "read", "read-write", "custom"]
    vpngrp: Literal["none", "read", "read-write"]
    utmgrp: Literal["none", "read", "read-write", "custom"]
    wanoptgrp: Literal["none", "read", "read-write"]
    wifi: Literal["none", "read", "read-write"]
    netgrp_permission: AccprofileNetgrppermissionObject
    sysgrp_permission: AccprofileSysgrppermissionObject
    fwgrp_permission: AccprofileFwgrppermissionObject
    loggrp_permission: AccprofileLoggrppermissionObject
    utmgrp_permission: AccprofileUtmgrppermissionObject
    secfabgrp_permission: AccprofileSecfabgrppermissionObject
    admintimeout_override: Literal["enable", "disable"]
    admintimeout: int
    cli_diagnose: Literal["enable", "disable"]
    cli_get: Literal["enable", "disable"]
    cli_show: Literal["enable", "disable"]
    cli_exec: Literal["enable", "disable"]
    cli_config: Literal["enable", "disable"]
    system_execute_ssh: Literal["enable", "disable"]
    system_execute_telnet: Literal["enable", "disable"]


# ================================================================
# Main Endpoint Class
# ================================================================

class Accprofile:
    """
    
    Endpoint: system/accprofile
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> AccprofileObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[AccprofileObject]: ...
    
    def get_schema(
        self,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: AccprofilePayload | None = ...,
        name: str | None = ...,
        scope: Literal["vdom", "global"] | None = ...,
        comments: str | None = ...,
        secfabgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        ftviewgrp: Literal["none", "read", "read-write"] | None = ...,
        authgrp: Literal["none", "read", "read-write"] | None = ...,
        sysgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        netgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        loggrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        fwgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        vpngrp: Literal["none", "read", "read-write"] | None = ...,
        utmgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        wanoptgrp: Literal["none", "read", "read-write"] | None = ...,
        wifi: Literal["none", "read", "read-write"] | None = ...,
        netgrp_permission: AccprofileNetgrppermissionDict | None = ...,
        sysgrp_permission: AccprofileSysgrppermissionDict | None = ...,
        fwgrp_permission: AccprofileFwgrppermissionDict | None = ...,
        loggrp_permission: AccprofileLoggrppermissionDict | None = ...,
        utmgrp_permission: AccprofileUtmgrppermissionDict | None = ...,
        secfabgrp_permission: AccprofileSecfabgrppermissionDict | None = ...,
        admintimeout_override: Literal["enable", "disable"] | None = ...,
        admintimeout: int | None = ...,
        cli_diagnose: Literal["enable", "disable"] | None = ...,
        cli_get: Literal["enable", "disable"] | None = ...,
        cli_show: Literal["enable", "disable"] | None = ...,
        cli_exec: Literal["enable", "disable"] | None = ...,
        cli_config: Literal["enable", "disable"] | None = ...,
        system_execute_ssh: Literal["enable", "disable"] | None = ...,
        system_execute_telnet: Literal["enable", "disable"] | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> AccprofileObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: AccprofilePayload | None = ...,
        name: str | None = ...,
        scope: Literal["vdom", "global"] | None = ...,
        comments: str | None = ...,
        secfabgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        ftviewgrp: Literal["none", "read", "read-write"] | None = ...,
        authgrp: Literal["none", "read", "read-write"] | None = ...,
        sysgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        netgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        loggrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        fwgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        vpngrp: Literal["none", "read", "read-write"] | None = ...,
        utmgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        wanoptgrp: Literal["none", "read", "read-write"] | None = ...,
        wifi: Literal["none", "read", "read-write"] | None = ...,
        netgrp_permission: AccprofileNetgrppermissionDict | None = ...,
        sysgrp_permission: AccprofileSysgrppermissionDict | None = ...,
        fwgrp_permission: AccprofileFwgrppermissionDict | None = ...,
        loggrp_permission: AccprofileLoggrppermissionDict | None = ...,
        utmgrp_permission: AccprofileUtmgrppermissionDict | None = ...,
        secfabgrp_permission: AccprofileSecfabgrppermissionDict | None = ...,
        admintimeout_override: Literal["enable", "disable"] | None = ...,
        admintimeout: int | None = ...,
        cli_diagnose: Literal["enable", "disable"] | None = ...,
        cli_get: Literal["enable", "disable"] | None = ...,
        cli_show: Literal["enable", "disable"] | None = ...,
        cli_exec: Literal["enable", "disable"] | None = ...,
        cli_config: Literal["enable", "disable"] | None = ...,
        system_execute_ssh: Literal["enable", "disable"] | None = ...,
        system_execute_telnet: Literal["enable", "disable"] | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> AccprofileObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: AccprofilePayload | None = ...,
        name: str | None = ...,
        scope: Literal["vdom", "global"] | None = ...,
        comments: str | None = ...,
        secfabgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        ftviewgrp: Literal["none", "read", "read-write"] | None = ...,
        authgrp: Literal["none", "read", "read-write"] | None = ...,
        sysgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        netgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        loggrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        fwgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        vpngrp: Literal["none", "read", "read-write"] | None = ...,
        utmgrp: Literal["none", "read", "read-write", "custom"] | None = ...,
        wanoptgrp: Literal["none", "read", "read-write"] | None = ...,
        wifi: Literal["none", "read", "read-write"] | None = ...,
        netgrp_permission: AccprofileNetgrppermissionDict | None = ...,
        sysgrp_permission: AccprofileSysgrppermissionDict | None = ...,
        fwgrp_permission: AccprofileFwgrppermissionDict | None = ...,
        loggrp_permission: AccprofileLoggrppermissionDict | None = ...,
        utmgrp_permission: AccprofileUtmgrppermissionDict | None = ...,
        secfabgrp_permission: AccprofileSecfabgrppermissionDict | None = ...,
        admintimeout_override: Literal["enable", "disable"] | None = ...,
        admintimeout: int | None = ...,
        cli_diagnose: Literal["enable", "disable"] | None = ...,
        cli_get: Literal["enable", "disable"] | None = ...,
        cli_show: Literal["enable", "disable"] | None = ...,
        cli_exec: Literal["enable", "disable"] | None = ...,
        cli_config: Literal["enable", "disable"] | None = ...,
        system_execute_ssh: Literal["enable", "disable"] | None = ...,
        system_execute_telnet: Literal["enable", "disable"] | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Accprofile",
    "AccprofilePayload",
    "AccprofileResponse",
    "AccprofileObject",
]


""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: authentication/scheme
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class SchemeUserdatabaseItem(TypedDict, total=False):
    """Nested item for user-database field."""
    name: str


class SchemePayload(TypedDict, total=False):
    """Payload type for Scheme operations."""
    name: str
    method: str | list[str]
    negotiate_ntlm: Literal["enable", "disable"]
    kerberos_keytab: str
    domain_controller: str
    saml_server: str
    saml_timeout: int
    fsso_agent_for_ntlm: str
    require_tfa: Literal["enable", "disable"]
    fsso_guest: Literal["enable", "disable"]
    user_cert: Literal["enable", "disable"]
    cert_http_header: Literal["enable", "disable"]
    user_database: str | list[str] | list[SchemeUserdatabaseItem]
    ssh_ca: str
    external_idp: str
    group_attr_type: Literal["display-name", "external-id"]
    digest_algo: str | list[str]
    digest_rfc2069: Literal["enable", "disable"]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class SchemeResponse(TypedDict, total=False):
    """Response type for Scheme - use with .dict property for typed dict access."""
    name: str
    method: str
    negotiate_ntlm: Literal["enable", "disable"]
    kerberos_keytab: str
    domain_controller: str
    saml_server: str
    saml_timeout: int
    fsso_agent_for_ntlm: str
    require_tfa: Literal["enable", "disable"]
    fsso_guest: Literal["enable", "disable"]
    user_cert: Literal["enable", "disable"]
    cert_http_header: Literal["enable", "disable"]
    user_database: list[SchemeUserdatabaseItem]
    ssh_ca: str
    external_idp: str
    group_attr_type: Literal["display-name", "external-id"]
    digest_algo: str
    digest_rfc2069: Literal["enable", "disable"]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class SchemeUserdatabaseItemObject(FortiObject[SchemeUserdatabaseItem]):
    """Typed object for user-database table items with attribute access."""
    name: str


class SchemeObject(FortiObject):
    """Typed FortiObject for Scheme with field access."""
    name: str
    method: str
    negotiate_ntlm: Literal["enable", "disable"]
    kerberos_keytab: str
    domain_controller: str
    saml_server: str
    saml_timeout: int
    fsso_agent_for_ntlm: str
    require_tfa: Literal["enable", "disable"]
    fsso_guest: Literal["enable", "disable"]
    user_cert: Literal["enable", "disable"]
    cert_http_header: Literal["enable", "disable"]
    user_database: FortiObjectList[SchemeUserdatabaseItemObject]
    ssh_ca: str
    external_idp: str
    group_attr_type: Literal["display-name", "external-id"]
    digest_algo: str
    digest_rfc2069: Literal["enable", "disable"]


# ================================================================
# Main Endpoint Class
# ================================================================

class Scheme:
    """
    
    Endpoint: authentication/scheme
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SchemeObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[SchemeObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: SchemePayload | None = ...,
        name: str | None = ...,
        method: str | list[str] | None = ...,
        negotiate_ntlm: Literal["enable", "disable"] | None = ...,
        kerberos_keytab: str | None = ...,
        domain_controller: str | None = ...,
        saml_server: str | None = ...,
        saml_timeout: int | None = ...,
        fsso_agent_for_ntlm: str | None = ...,
        require_tfa: Literal["enable", "disable"] | None = ...,
        fsso_guest: Literal["enable", "disable"] | None = ...,
        user_cert: Literal["enable", "disable"] | None = ...,
        cert_http_header: Literal["enable", "disable"] | None = ...,
        user_database: str | list[str] | list[SchemeUserdatabaseItem] | None = ...,
        ssh_ca: str | None = ...,
        external_idp: str | None = ...,
        group_attr_type: Literal["display-name", "external-id"] | None = ...,
        digest_algo: str | list[str] | None = ...,
        digest_rfc2069: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SchemeObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: SchemePayload | None = ...,
        name: str | None = ...,
        method: str | list[str] | None = ...,
        negotiate_ntlm: Literal["enable", "disable"] | None = ...,
        kerberos_keytab: str | None = ...,
        domain_controller: str | None = ...,
        saml_server: str | None = ...,
        saml_timeout: int | None = ...,
        fsso_agent_for_ntlm: str | None = ...,
        require_tfa: Literal["enable", "disable"] | None = ...,
        fsso_guest: Literal["enable", "disable"] | None = ...,
        user_cert: Literal["enable", "disable"] | None = ...,
        cert_http_header: Literal["enable", "disable"] | None = ...,
        user_database: str | list[str] | list[SchemeUserdatabaseItem] | None = ...,
        ssh_ca: str | None = ...,
        external_idp: str | None = ...,
        group_attr_type: Literal["display-name", "external-id"] | None = ...,
        digest_algo: str | list[str] | None = ...,
        digest_rfc2069: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SchemeObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: SchemePayload | None = ...,
        name: str | None = ...,
        method: Literal["ntlm", "basic", "digest", "form", "negotiate", "fsso", "rsso", "ssh-publickey", "cert", "saml", "entra-sso"] | list[str] | None = ...,
        negotiate_ntlm: Literal["enable", "disable"] | None = ...,
        kerberos_keytab: str | None = ...,
        domain_controller: str | None = ...,
        saml_server: str | None = ...,
        saml_timeout: int | None = ...,
        fsso_agent_for_ntlm: str | None = ...,
        require_tfa: Literal["enable", "disable"] | None = ...,
        fsso_guest: Literal["enable", "disable"] | None = ...,
        user_cert: Literal["enable", "disable"] | None = ...,
        cert_http_header: Literal["enable", "disable"] | None = ...,
        user_database: str | list[str] | list[SchemeUserdatabaseItem] | None = ...,
        ssh_ca: str | None = ...,
        external_idp: str | None = ...,
        group_attr_type: Literal["display-name", "external-id"] | None = ...,
        digest_algo: Literal["md5", "sha-256"] | list[str] | None = ...,
        digest_rfc2069: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Scheme",
    "SchemePayload",
    "SchemeResponse",
    "SchemeObject",
]


"""FortiOS CMDB - Vdom category"""

from .add_license import AddLicense

__all__ = [
    "AddLicense",
    "Vdom",
]


class Vdom:
    """Vdom endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Vdom endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.add_license = AddLicense(client)



"""Type stubs for SESSION category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .close import Close
    from .close_all import CloseAll
    from .close_multiple import CloseMultiple

__all__ = [
    "Close",
    "CloseAll",
    "CloseMultiple",
    "Session",
]


class Session:
    """SESSION API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    close: Close
    close_all: CloseAll
    close_multiple: CloseMultiple

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize session category with HTTP client."""
        ...



"""FortiOS CMDB - VlanProbe category (stub)"""

from typing import Any
from ..vlan_probe_base import VlanProbe as VlanProbeBase
from .start import Start
from .stop import Stop

class VlanProbe(VlanProbeBase):
    """VlanProbe endpoints wrapper for CMDB API."""

    start: Start
    stop: Stop

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
